#!/usr/bin/python
# -*- coding: UTF-8 -*-

'''
Created on 3.3.2011

@author: olkauppi
'''

import zipfile
import xml.dom.minidom
import conversions
import string
import ooo_meta_inspector




def makeDict(filename):
    '''For testing...'''
    
    #filename = sys.argv[1]

    
    
    documentDict = {}
    if zipfile.is_zipfile(filename):
        zip=zipfile.ZipFile(filename)
    
        for data in zip.filelist:
            try:
                documentDict[data.filename]=xml.dom.minidom.parseString(zip.read(data.filename))
            except:
                documentDict[data.filename]=zip.read(data.filename)
        return documentDict
    #                 
    #      
    #    
    #myodf = zipfile.ZipFile(filename)
    #contentXml = myodf.read('content.xml')
    #documentDict['content.xml'] = xml.dom.minidom.parseString(contentXml)
    #stylesXml = myodf.read('styles.xml')
    #documentDict['styles.xml'] = xml.dom.minidom.parseString(stylesXml)
    #metaXml = myodf.read('meta.xml')
    #documentDict['meta.xml'] = xml.dom.minidom.parseString(metaXml)
    
 


def getStyleElementByStyleName (documentDict,styleName):
    '''Searches first from content.xml if doesn't find then searches from
    style.xml. 
    Return element if style exists if doesn't return None.   
    '''
    styleElements =  documentDict['content.xml'].getElementsByTagName('style:style')
    for element in styleElements:
        if element.getAttribute('style:name')==styleName:
            return element
                
    styleElements =  documentDict['styles.xml'].getElementsByTagName('style:style')
    for element in styleElements:
        if element.getAttribute('style:name')==styleName:
            return element
    return None  

def getStyleDisplayNameByStyleName (documentDict, styleName):
    '''Uses getStyleElementByStyleName-method to find the style.
    If style don't have display-name -attribute then display name is 
    just style name. 
    return style display name.
    
    Example:
    <style:style style:name="Text_20_body" style:display-name="Text body">
    <style:style style:name="tutkielma">
    
    Display name --> style name
    " "-->"_20_"
    "_"-->"_5f_"   
    
    '''
    
    element = getStyleElementByStyleName(documentDict,styleName)
    if element.hasAttribute('style:display-name'):
        return element.getAttribute('style:display-name')
    else:
        return styleName
    
def getMasterPageStyleElement (documentDict,masterPageStyleName):
    '''Return master page -element with given master page name.    
    '''
    masterPageStyles = documentDict['styles.xml'].getElementsByTagName ('style:master-page')
    for element in masterPageStyles:        
        if masterPageStyleName == element.getAttribute('style:name'):
            return element
    return None
        
def getPageLayoutElement(documentDict,pageLayoutName):
    '''Return page layout -element with given page layout name.
    '''
    pageLayoutElements = documentDict['styles.xml'].getElementsByTagName ('style:page-layout')
    for element in pageLayoutElements:
        if pageLayoutName == element.getAttribute('style:name'):
            return element
    return None

def printPageLayoutProperties(pageLayoutElement):
    '''Print page layout properties by given page layout -element.
    '''
    pageLayoutProperties = pageLayoutElement.getElementsByTagName('style:page-layout-properties')[0]
    print pageLayoutElement.getAttribute('style:name')
    print "leveys: ", pageLayoutProperties.getAttribute('fo:page-width')
    print "korkeus: ", pageLayoutProperties.getAttribute('fo:page-height')
    print "Marginaalit:"
    print "yla: ", pageLayoutProperties.getAttribute('fo:margin-top')
    print "ala: ", pageLayoutProperties.getAttribute('fo:margin-bottom')
    print "vasen: ", pageLayoutProperties.getAttribute('fo:margin-left')
    print "oikea: ", pageLayoutProperties.getAttribute('fo:margin-right')
        
def getPageMarginals(documentDict):
    '''Return page marginals. If marginals are different between used pages,
    then return false.
    '''
    pageLayoutElements = documentDict['styles.xml'].getElementsByTagName('style:page-layout')
    margins = {'top': None,'bottom': None,'right': None,'left': None}
    for element in pageLayoutElements:
        for key in margins.keys():            
            margin = element.getElementsByTagName('style:page-layout-properties')[0].getAttribute('fo:margin-'+key)
            
            if margins [key] is None:
                margins [key] = margin
            elif margins [key] != margin:   
                return False
            
                
            
    return conversions.convertCmDictToFloatDict(margins)

def getPageSize(documentDict):
    '''Return page size. If size is different between used pages,
    then return error message.
    '''
    pageLayoutElements = documentDict['styles.xml'].getElementsByTagName('style:page-layout')
    pageSize = {'height': None,'width': None }
    for element in pageLayoutElements:
        for key in pageSize.keys():            
            size = element.getElementsByTagName('style:page-layout-properties')[0].getAttribute('fo:page-'+key)
            
            if pageSize [key] is None:
                pageSize [key] = size
            if pageSize [key] != size:
                return False
            
    return conversions.convertCmDictToFloatDict(pageSize)


    
    
def getDefaultStyleElement(documentDict,styleFamily):
    '''Style family tells which default style is wanted.
    Return default style element.
    
    Style family:
    paragraph, graphic, table and table-row.
    
    Every style is based on style family. 
    <style:style style:name="Heading_20_1" style:display-name="Heading 1" style:family="paragraph">
    '''
    defaultStyleElements = documentDict['styles.xml'].getElementsByTagName('style:default-style')
    for element in defaultStyleElements:
        if element.getAttribute ('style:family')==styleFamily:
            return element
    return None



def getElementFromContent(documentDict,elementName):
    return documentDict['content.xml'].getElementsByTagName (elementName)

def getMasterPageStyleName(styleElement):
    '''Return master page name by style element.
    '''
    masterPageName = styleElement.getAttribute ('style:master-page-name')
    if masterPageName == '':
        return 'Standard'
    else:
        return masterPageName


def findEmptyParagraphs(documentDict):
    '''
    '''
    
    paragraphs = documentDict['content.xml'].getElementsByTagName('text:p')
    emptyParagraphs = 0
    for element in paragraphs:
        if not element.childNodes:
            emptyParagraphs = emptyParagraphs + 1
    paragraphs =documentDict['content.xml'].getElementsByTagName('text:h')
    for element in paragraphs:
        if not element.childNodes:
            emptyParagraphs = emptyParagraphs + 1
    return emptyParagraphs
    
def getListOfUsedStyleElements (documentDict):
    '''Lists all used styles in document.
    Content.xml->office:body-element contains used styles.
    return element list of used styles.
    '''
    
    
    usedStyleElements = []
    
    
    
    bodyElements = getDescendants(documentDict['content.xml'].getElementsByTagName('office:body')[0],[])
    
    for element in bodyElements:
        if element.nodeType is not element.TEXT_NODE and element.hasAttribute('text:style-name'):
            usedStyleElements.append(element)
            
#    paragraphs = documentDict['content.xml'].getElementsByTagName('text:p')    
#    for element in paragraphs:      
#        usedStyles.append(element.getAttribute('text:style-name'))
#
#    headings = documentDict['content.xml'].getElementsByTagName('text:h')
#    for element in headings:
#        usedStyles.append(element.getAttribute('text:style-name'))
#
#    lists = documentDict['content.xml'].getElementsByTagName('text:list')
#    for element in lists:
#        usedStyles.append(element.getAttribute('text:style-name'))

    return usedStyleElements

def getListOfUsedStyleNames ():
    usedStyleNames = []
    for element in getListOfUsedStyleElements (documentDict):
        if element.nodeName == 'text:p' or element.nodeName == 'text:h':
            usedStyleNames.append(element.getAttribute('text:style-name'))
    
    usedStyleNames=list(set(usedStyleNames)) #deletes duplicates from list
    usedStyleNames.sort()
#    print "kaytetyt tyylit (lkm):", len(usedStyleNames)
#    print "Kaytetyt tyylit:", ", ".join(usedStyleNames)
    return usedStyleNames
    
    
def checkPageBreakStyleElement (styleElement):
    '''Return style element if style breaks the page. 
    '''
    
    hasParagraphProperties = styleElement.getElementsByTagName('style:paragraph-properties')
    if hasParagraphProperties:
        if hasParagraphProperties[0].getAttribute('fo:break-before') == 'page':
            return styleElement
    return False 

def getPageBreakStyleNames(documentDict):
    '''Lists all styles which changes page.
    return element list.    
    '''
    pageBreakStylesElements = []
    styleElements = documentDict['content.xml'].getElementsByTagName('style:style')
    styleElements += documentDict['styles.xml'].getElementsByTagName('style:style')
    for element in styleElements:
        pageBreak = checkPageBreakStyleElement(element)
        if pageBreak is not False:
            pageBreakStylesElements.append(element.getAttribute('style:name'))
      
    
    #print "sivua vaihtavat tyylit:", ", ".join(pageBreakStylesElements)
    return pageBreakStylesElements

def getAllStyleNamesWithDifferentMasterPage(documentDict):
    '''Lists all style names which changes master page.
    Master page will change when style has master-page-name -attribute
    and attribute value is not empty. If is empty then master page is same than
    previous style.
    Return dict of styles which changes master page.
    '''
    styleElements = documentDict['content.xml'].getElementsByTagName('style:style')
    styleElements += documentDict['styles.xml'].getElementsByTagName('style:style')
    
    masterPageDict = {}
    
    for element in styleElements:
        if element.getAttribute('style:master-page-name') !='':
            masterPageDict[element.getAttribute('style:name')] = element.getAttribute('style:master-page-name')
    return masterPageDict
    
def getSectionBreakElements(documentDict):
    '''Get section break elements from document. Finds elements (text,list, heading...) 
    which chance section. 
    Return list of elements which changes section    
    '''
    usedStyleElementsInDocument = getListOfUsedStyleElements(documentDict)
    masterPagesDict = getAllStyleNamesWithDifferentMasterPage(documentDict)
    sectionBreakElements = []
    for element in usedStyleElementsInDocument:
        if masterPagesDict.has_key(element.getAttribute('text:style-name')):
            sectionBreakElements.append(element)
    return sectionBreakElements

    
def getDescendants (element,elementList): #common_methods
    '''Get elements all descendants.
    element is the node which descendants have to find
    elementList is empty list
    return list of all descendants
    '''
    elementChildren = element.childNodes
    
    for element in elementChildren:
        elementList.append(element)
        if element.hasChildNodes():
            getDescendants(element,elementList)
    return elementList
        

def getTOC(documentDict):
    '''Each TOC entry is own entry in tocList.
    Return list of table of content.     
    '''
    tocList = []
    if checkTOC(documentDict) is True:
        toc = documentDict['content.xml'].getElementsByTagName('text:table-of-content')
        tocTextElements = toc[0].getElementsByTagName ('text:p')
        for element in tocTextElements:
            if element.parentNode.nodeName == 'text:index-title':
                tocTitle = getTextContent(element)
                #getprint tocTitle
            else:
                tocList.append(getTextContent(element))
            #print getTextContent(element)
        return tocList
   
def checkTocContent(documentDict):
    '''Compare document headings to TOC entries. 
    Return True if all entries matches otherwise return error message.
    '''
    tocList = getTOC(documentDict)
    headingList = getHeadingList(documentDict)['headings']

    if len(tocList)!=len(headingList):
        return False#"Sisällysluettelon otsikkoja on eri määrä dokumentin otsikkoja"
    i=0 
    while i<len(tocList):
        if string.find (tocList[i],headingList[i])==-1:
            return False #headingList[i-1]+" -otsikkoa ei löytynyt sisällysluettelosta"
        i+=1
    return True
    
    
        
def checkTOC(documentDict):
    '''Check if document contains table of contents.
    return true if there is table of contents
    return error message if not. 
    '''
    toc = documentDict['content.xml'].getElementsByTagName('text:table-of-content')
    if len(toc)==0:
        return False
    else:
        return True
    
def checkIndex(documentDict):
    '''Check if document have alphabetical index.
    Return True if alphabetical index exists otherwise return error message.
    '''
    aIndex = documentDict['content.xml'].getElementsByTagName('text:alphabetical-index')
    if len(aIndex)==0: return False    
    return True

def getIndexContentFromDocument (documentDict):
    list = []
    indexMarks = documentDict['content.xml'].getElementsByTagName ('text:alphabetical-index-mark-start')
    for element in indexMarks:
        list.append (element.nextSibling.nodeValue)
    return list




    
def getIndexContent(documentDict):
    '''Each alphabetical index entry is own entry in list.
    Return list of alphabetical index content.
    '''
    alphabeticalIndexList = [] 
    if checkIndex(documentDict) is True:
        aIndex = documentDict['content.xml'].getElementsByTagName('text:alphabetical-index')[0]
        aIndexTextElements = aIndex.getElementsByTagName('text:p')
        for element in aIndexTextElements:
            alphabeticalIndexList.append(getTextContent(element))
    return alphabeticalIndexList       

def checkIndexContent():
    '''Compare document marked texts to Index entries. 
    Return True if all entries matches otherwise return error message.
    '''
    indexList = getIndexContent(documentDict)
    markedList = getIndexContentFromDocument(documentDict)

    for markedItem in markedList:
        for indexItem in indexList:
            found = False
            if string.find (indexItem,markedItem)!=-1:
                found = True
                break  
        if found is False:
            return False
           
    return True



    


def getHeadingList(documentDict):
    '''Each heading is own entry in list.
    Return list of headings.
    '''
    headingList = []
    headingOutlineLevel = 0
    headings = documentDict['content.xml'].getElementsByTagName('text:h')
    #print "Dokumentin otsikot:"
    for element in headings:
        heading = getTextContent(element)
        headingList.append(heading)
        if headingOutlineLevel < element.getAttribute('text:outline-level'):
            headingOutlineLevel = element.getAttribute('text:outline-level')
        
        #print " -",heading, "(",headingOutlineLevel,"-tason otsikko)"
    return {'headings':headingList,'level':headingOutlineLevel}
    
def checkTable (documentDict):
    '''Check if document has a table.
    '''
    table = documentDict['content.xml'].getElementsByTagName('table:table')
    if len(table)==0:
        return "There is no table at all."
    else:
        return True
    

def getTableDict (documentDict):
    '''Get table in dict.
    Return table.
     
    first table entry is A1.
    Cell address(A1,B1,B5) is dicts key value and cell value is 
    dicts value. [['A1','Otsikko'],['B5','120']]
    '''
    if checkTable (documentDict) is True:
        tableDict = {}
        tableElement = documentDict['content.xml'].getElementsByTagName('table:table')[0]
        tableRowElements = tableElement.getElementsByTagName('table:table-row')
        rowIndex=0
        for row in tableRowElements:
            rowIndex +=1
            rowCellElements = row.getElementsByTagName('table:table-cell')
            columnIndex = ord('A')-1
            for cell in rowCellElements:
                columnIndex +=1
                index = chr(columnIndex) + str(rowIndex)
                tableDict[index] = getTextContent(cell)
        return tableDict
    return "There is no table at all"
                
            

def findStringFromParagraphs(findString,element): #common_methods
    
        
    paragraphsText = getTextContent(element)#.encode('utf-8')
    findString#.encode('utf-8')
    
    if paragraphsText.find(findString)==-1:
        return False
    else:
        return True
            

def checkPageNumberFromFooterAndHeader(masterPageElement, element):
    '''Element is footer or header element. 
    Return number format if it exists otherwise return False.
    
    Number format is optionally in element (footer or header). If there
    is undefined in element then page-layout defines number format.
    '''
    pageLayoutElement = getPageLayoutElement(documentDict,masterPageElement.getAttribute('style:page-layout-name'))
    pageNumberElements = element.getElementsByTagName('text:page-number')
    if pageNumberElements:
        if pageNumberElements[0].hasAttribute('style:num-format'):
            numFormat = pageNumberElements[0].getAttribute('style:num-format')
        else:
            numFormat = pageLayoutElement.getElementsByTagName('style:page-layout-properties')[0].getAttribute('style:num-format')
        return numFormat
    
    return False


    
    
    

def checkAuthorAndPageNumber(masterPageElement):       
    print masterPageElement.getAttribute('style:name')
    pageNumber = False
    author =False
    meta = ooo_meta_inspector.getMeta(documentDict)
    footer = masterPageElement.getElementsByTagName ('style:footer')
    header = masterPageElement.getElementsByTagName ('style:header')         
    if footer:
        pageNumber = checkPageNumberFromFooterAndHeader(masterPageElement, footer[0])   
        if findStringFromParagraphs(meta['dc:creator'],footer[0]):
            author = meta['dc:creator']
        print "ylatuniste:"
        print getTextContent(footer[0])
    if header:
        pageNumber = checkPageNumberFromFooterAndHeader(masterPageElement, header[0])
        if findStringFromParagraphs(meta['dc:creator'],footer[0]):
            author = meta['dc:creator']
        print "alatuniste:"
        print getTextContent(header[0])
    
    if pageNumber is False:
        print 'sivunumeroa ei löytynyt'
    else:
        print 'sivunumero löytyi, formaatti:', pageNumber
    
    if author is False:
        print 'tekijännimeä ei löytynyt'
    else:
        print 'tekijä löytyi:', author        
    

def checkHeadingNumbering(documentDict, errorIdsAndPositions):
    '''Check outline style.
    level is highest used headings outline level. Normally Heading 1 --> 1, Heading 2 --> 2
    return True if ok, False if not.
    '''
    
    '''XML:
    <text:outline-style style:name="Outline">
        <text:outline-level-style text:level="1" style:num-format="1">
            <style:list-level-properties text:list-level-position-and-space-mode="label-alignment">
                <style:list-level-label-alignment text:label-followed-by="listtab" text:list-tab-stop-position="0.762cm" fo:text-indent="-0.762cm" fo:margin-left="0.762cm"/>
            </style:list-level-properties>
        </text:outline-level-style>

        <text:outline-level-style text:level="2" style:num-format="1" text:display-levels="2">
            <style:list-level-properties text:list-level-position-and-space-mode="label-alignment">
                <style:list-level-label-alignment text:label-followed-by="listtab" text:list-tab-stop-position="1.016cm" fo:text-indent="-1.016cm" fo:margin-left="1.016cm"/>
            </style:list-level-properties>
        </text:outline-level-style>

        <text:outline-level-style text:level="3" style:num-format="">
            <style:list-level-properties text:list-level-position-and-space-mode="label-alignment">
                <style:list-level-label-alignment text:label-followed-by="listtab" text:list-tab-stop-position="1.27cm" fo:text-indent="-1.27cm" fo:margin-left="1.27cm"/>
            </style:list-level-properties>
        </text:outline-level-style>   
        .
        .
    </text:outline-style>
    '''
    outlineStyleElement = documentDict['styles.xml'].getElementsByTagName('text:outline-style')
    outlineLevels = outlineStyleElement[0].getElementsByTagName('text:outline-level-style')
    level = getHeadingList(documentDict)['level']
    index = 0
    for element in outlineLevels:  
        index = index + 1
        if index>int(level):
            return True         
        if int(element.getAttribute('text:level'))==index and element.getAttribute('style:num-format') != "":
            "ok"#print index,"-tason otsikossa numerointi kunnossa"
        else:
            return False#"numerointi ei yllä "+ str(level)+ "-tasoon"

    



def getImagePaths(documentDict):
    '''Searches if document have an image.
    return images path if founds an image otherwise return error message.
    '''
    imagePathList =[]
    if checkImages(documentDict) is True:
        imageElements = documentDict['content.xml'].getElementsByTagName ('draw:image')
        for element in imageElements:
            imagePath = element.getAttribute ('xlink:href')
            imagePathList.append(imagePath)
        
    return imagePathList  

        
def checkImages(documentDict):
    '''Check if document contains an image.
    return true if there is an image otherwise
    return error message. 
    '''
    imageElements = documentDict['content.xml'].getElementsByTagName ('draw:image')
    if len(imageElements) == 0:
        return False
    else:
        return True
    
def checkList(documentDict):
    '''Check if document contains list.
    return true if there is a list
    return error message if not. 
    '''
    listElements = documentDict['content.xml'].getElementsByTagName ('text:list')
    if len(listElements) == 0:
        return False
    
    return True

def printLists(documentDict):
    '''Print document lists
    
    '''
    if checkList(documentDict) is True:
        listElements = documentDict['content.xml'].getElementsByTagName ('text:list')
        for list in listElements:
            print 'Lista tehty', list.getAttribute('text:style-name'),'-tyylillä, lista:'
            listContent = list.getElementsByTagName ('text:list-item')
            for text in listContent:
                print "-",getTextContent(text)
    
    
def getObjectPaths(documentDict):
    '''Searches if document have an image.
    return object path list if founds an image if not return error
    '''
    objectPathList=[]
    objectElements = documentDict['content.xml'].getElementsByTagName ('draw:object')
    if objectElements:
        for element in objectElements:
            objectPath = element.getAttribute ('xlink:href')
            objectPathList.append(objectPath)  
        return objectPathList 
    else:
        return "VIRHE: objektia ei loytynyt"
    
#TODO: sijoitus pitää poistaa ja tehdä jotenkin toisin
def checkObject(documentDict,objectPath):
    documentDict['content.xml']=documentDict[objectPath+'/content.xml']
    checkTable(documentDict)
    getTableDict(documentDict)
    print objectPath+'/content.xml'

def getStyle(documentDict, styleName): 
    '''Prints styles attributes
    parentStyleList is for executing the inheritation of styles.
    Inheritation of styles:
      default paragraph -style-> standard-style -> style(Text body) -> P-style -> T-style
      
    XML (styles.xml):
    
    <style:style style:name="Standard" style:family="paragraph" style:class="text">
        <style:paragraph-properties fo:orphans="2" fo:widows="2" style:writing-mode="lr-tb"/>
        <style:text-properties style:use-window-font-color="true" style:font-name="Courier New" fo:font-size="10pt" fo:language="fi" fo:country="FI" style:font-name-asian="Times New Roman" style:font-size-asian="10pt" style:font-name-complex="Times New Roman" style:font-size-complex="10pt" style:language-complex="ar" style:country-complex="SA"/>
    </style:style>
    
    <style:style style:name="Text_20_body" style:display-name="Text body" style:family="paragraph" style:parent-style-name="Standard" style:class="text" style:master-page-name="">
        <style:paragraph-properties fo:margin-left="1cm" fo:margin-right="0cm" fo:margin-top="0.247cm" fo:margin-bottom="0.247cm" fo:text-indent="0cm" style:auto-text-indent="false" style:page-number="auto" fo:break-before="auto" fo:break-after="auto"/>
        <style:text-properties style:font-name="Tahoma"/>
    </style:style
    
    XML (content.xml):
    
    <style:style style:name="P2" style:family="paragraph" style:parent-style-name="Text_20_body">
        <style:paragraph-properties fo:text-align="start" style:justify-single-word="false"/>
    </style:style>
    
    '''   
    if getStyleElementByStyleName(documentDict,styleName) is None:
        return False  
    #if getStyleElementByStyleName(documentDict, getStyleDisplayNameByStyleName(documentDict, styleName).lower()) is None:
    #   re
    
    defaultStyleElement = getDefaultStyleElement(documentDict,'paragraph')

    fontName = defaultStyleElement.getElementsByTagName('style:text-properties')[0].getAttribute('style:font-name')
    fontSize = defaultStyleElement.getElementsByTagName('style:text-properties')[0].getAttribute('fo:font-size')
    language = defaultStyleElement.getElementsByTagName('style:text-properties')[0].getAttribute('fo:language')
        
    
    styleAttributeDict = {
    'style:name':styleName,
    'style:parent-style-name':None,
    'fo:language':language,
    'style:font-name':fontName,
    'fo:font-size':fontSize,
    'fo:text-transform':None, 
    'fo:text-indent':None, 
    'fo:line-height':'100%',
    'fo:margin-left':0,
    'fo:margin-right':0,
    'fo:margin-top':0,
    'fo:margin-bottom':0, 
    'fo:keep-with-next':False, 
    'fo:text-align': 'start', 
    'fo:orphans':None, 
    'fo:widows': None,
    'fo:font-style': None,
    'fo:font-weight':None}
    
    translateDict = {
    'style:name': 'styleName',
    'style:font-name':'fontName',
    'fo:font-size':'fontSize',
    'fo:text-transform':'transform', 
    'fo:text-indent':'indent', 
    'fo:line-height':'linespacing',
    'fo:margin-top':'spacingBefore',
    'fo:margin-bottom':'spacingAfter', 
    'fo:keep-with-next':'keepWithNext', 
    'fo:text-align': 'alignment', 
    'fo:font-style': 'bold',
    'fo:font-weight':'italic',
    'widowControl':'widowControl'
    }
    styleDict = {}
    
    parentStyleList = getParentStyleList (documentDict,styleName)
    
    #Get style attribute values
    while parentStyleList:
        styleAttributeDict['style:name']= parentStyleList.pop()
        styleAttributeDict = getStyleAttributes (documentDict, styleAttributeDict)
    
    #Conversions:   
    styleAttributeDict['fo:margin-top'] = conversions.convertCmToPt(styleAttributeDict['fo:margin-top'])
    styleAttributeDict['fo:margin-bottom'] =  conversions.convertCmToPt(styleAttributeDict['fo:margin-bottom'])
    styleAttributeDict['style:name']= getStyleDisplayNameByStyleName(documentDict, styleAttributeDict['style:name']).lower()
    styleAttributeDict['fo:line-height'] =  conversions.convertPercentToDecimal(styleAttributeDict['fo:line-height'])
    styleAttributeDict['fo:font-size'] = styleAttributeDict['fo:font-size'].split('pt')[0]
    styleAttributeDict['fo:text-indent']= styleAttributeDict['fo:text-indent'].split('cm')[0]
    
    if styleAttributeDict['fo:widows']>=2 and styleAttributeDict['fo:orphans']>=2:
        styleAttributeDict['widowControl']= 1 #same in docx
    else:
        styleAttributeDict['widowControl'] = 0 #same in docx
    
    
    #Translate keys
    for key in translateDict.keys():
        styleDict[translateDict[key]] = styleAttributeDict[key]
        
    return styleDict
        

 
#TODO: omaan aliohjelmaan atribuuttien arvojen haku getAttribute(element,styleAttributeList)   
def getStyleAttributes (documentDict, styleAttributeList):
    '''Searches if style have wanted attribute if have then replace value.
    Style attribute list contains all relevant style information
    Return list of styles attributes    
    '''
    styleElement = getStyleElementByStyleName(documentDict, styleAttributeList['style:name'])
       
    if styleElement:
        paragraphPropertiesElement = styleElement.getElementsByTagName('style:paragraph-properties')
        textPropertiesElement=styleElement.getElementsByTagName('style:text-properties')
    
        for attribute in styleAttributeList.keys():
            if styleElement.hasAttribute (attribute): 
                styleAttributeList[attribute] = styleElement.getAttribute (attribute)
    
        if paragraphPropertiesElement:        
            for attribute in styleAttributeList.keys():            
                if paragraphPropertiesElement[0].hasAttribute (attribute):
                    styleAttributeList[attribute] = paragraphPropertiesElement[0].getAttribute (attribute)
    
        if textPropertiesElement:
            for attribute in styleAttributeList.keys():
                if textPropertiesElement[0].hasAttribute (attribute):
                    styleAttributeList[attribute] = textPropertiesElement[0].getAttribute (attribute)
        
    return styleAttributeList


    
def getParentStyleList (documentDict, styleName):
    '''Lists styles parent styles
    returns a list of parent styles (lists first entry is style itself)
    '''
    parentStyleList=[styleName]
       
    while checkParentStyle(documentDict,styleName):
        styleName = checkParentStyle(documentDict,styleName)
        parentStyleList.append(styleName)
    
    
    return parentStyleList


def checkParentStyle (documentDict,styleName):
    '''Checks if style have a parent style
    Returns parent style
    '''
    try:
        return getStyleElementByStyleName(documentDict,styleName).getAttribute('style:parent-style-name')
    except:
        return None
  
def checkEndnotesAndFootnotes(documentDict):
    '''Check end- and footnotes.
    
    '''
    noteElements = getElementFromContent(documentDict,'text:note')
    if len(noteElements)==0:
        return False
#        for element in noteElements:
#            if element.getAttribute ('text:note-class')=='endnote':
#                return True
#            elif element.getAttribute ('text:note-class')=='footnote':
#                return True
    return True
      
def getImageCaption(documentDict):
    '''Get image caption.
    
    '''
    imageElements = documentDict['content.xml'].getElementsByTagName ('draw:image')
    if imageElements:
        for element in imageElements:
            captionNode = element.parentNode.parentNode
            print "kuvateksti:", getTextContent(captionNode) #.lastChild.nodeValue
            imagesReferenceElements = captionNode.getElementsByTagName('text:sequence')
            if imagesReferenceElements:
                imagesReference = imagesReferenceElements[0].getAttribute('text:ref-name')
                print "kuvaviite:",imagesReference
                
#TODO: ei välttämättä tarvita           
def printStyleAttributes(documentDict,styleName):
    '''Common style. DOES NOT WORK.
    '''
    
    defaultStyleElements = documentDict['styles.xml'].getElementsByTagName('style:default-style')
    for element in defaultStyleElements:
        if element.getAttribute ('style:family') == 'paragraph':
            defaultStyleElement = element

    fontName = defaultStyleElement.getElementsByTagName('style:text-properties')[0].getAttribute('style:font-name')
    fontSize = defaultStyleElement.getElementsByTagName('style:text-properties')[0].getAttribute('fo:font-size')
    language = defaultStyleElement.getElementsByTagName('style:text-properties')[0].getAttribute('fo:language')
    listStyleName = None
       
    parentStyleList = getParentStyleList (styleName)
    
    styleAttributeList = {'style:name':styleName,'fo:language':language,'style:font-name':fontName,'fo:font-size':fontSize,'style:list-style-name':listStyleName}
    
    while parentStyleList:            
        styleAttributeList['style:name']= parentStyleList.pop()
        styleAttributeList = getStyleAttributes (documentDict, styleAttributeList)
    
                    
    for i in styleAttributeList.keys():
        print i, ":" ,styleAttributeList[i]

def getTextContent (element): #common_methods
    '''Get element text content.
    return all text content from element and descendants.
    '''
    elementsChildList = element.childNodes
    content = ''
    
    for childElement in elementsChildList:
        if childElement.nodeType== childElement.TEXT_NODE:
            content += childElement.nodeValue
        else:
            content += getTextContent(childElement)
    return content

def checkCoverPage(documentDict):
    '''  Checks if the front page is done correctly ̈́
    
    return 1 -- if email is not found
    return 2 -- if name is not found
    return True -- if everything is OK
    
    '''
    elementList = getCoverPageElements(documentDict)
    if elementList is None:
        return False
    meta = ooo_meta_inspector.getMeta(documentDict)
    cover = {
    'title':False,
    'name':False,
    'email' :False
    }
    for element in elementList:        
        if checkIfEmailAddress (element):
            cover['email'] = True
        if findStringFromParagraphs(meta['dc:creator'],element):
            cover['name']=True
        if findStringFromParagraphs(meta['dc:title'],element):
            cover['title'] = True
    return cover
                    
#        if element.nodeName == 'text:table-of-content':
#            print "VIRHE: kansilehti ja sisällysluettelo samassa osassa"
#        elif element.nodeName == 'draw:frame':
#            print "VIRHE: kansilehdellä ylimääräinen kehys"
#        elif element.nodeName == 'table:table':
#            print "VIRHE: kansilehdellä taulukko"
#    
#    if eMail is False:   
#        print 'VIRHE: tekijän s-postia ei löytyny kannesta'
#    else:   
#        print "s-posti ("+ eMail+") löyti kannesta"
#        
#    if author is False:
#        print 'VIRHE: tekijän nimeä ei löytyny kannesta'
#    else:
#        print "tekijä (", author,") löytyi kannesta"
#        
#    if title is False:
#        print 'VIRHE: titleä ei löytyny kannesta'
#    else:
#        print "Title ("+ title+") löytyi kannesta" 
        
       


def getSectionElements (documentDict,section):
    '''Get elements of wanted section.
    Section break elements changes section.
    First list elements to cover-section.
    Second list elements to toc-section.
    And last list element to text-section.
    Document have to have atleast 3 sections.
    Return sections elements in list.
    '''
    sectionBreakElementList = getSectionBreakElements(documentDict)
    sectionElements = {'cover':None,'toc':None,'text':None}
    sectionList = []
    officeBodyElement = documentDict['content.xml'].getElementsByTagName ('office:body')[0] #always exact 1 element
    documentElementList = officeBodyElement.firstChild.childNodes
    i = 0
    k = len(documentElementList)
    if len(sectionBreakElementList) < 3:
        #print "dokumentissa liian vähän osia"
        return None
    while not documentElementList[i].isSameNode(sectionBreakElementList[1]):
        sectionList.append(documentElementList[i])
        i+=1
    sectionElements ['cover'] = sectionList 
    sectionList = []
    while not documentElementList[i].isSameNode(sectionBreakElementList[2]):
        sectionList.append(documentElementList[i])
        i+=1
    sectionElements ['toc'] = sectionList
    sectionList = []
    while i<k:
        sectionList.append(documentElementList[i])
        i+=1
    sectionElements ['text'] = sectionList
        
        
    
    return sectionElements[section]

        

def getCoverPageElements(documentDict):
    elementList = getSectionElements(documentDict, 'cover')
    
    return elementList
      
def getTocPageElements ():
    elementList = getSectionElements('toc')
    return elementList

def getTextPageElements ():
    elementList = getSectionElements('text')
    return elementList
    

    
def checkSectionAmount():
    '''Count section breaks...
    Return True...
    '''
    sections = len(getSectionBreakElements ())
    if sections < 3:
        return False
    else:
        return True
    
def checkIfEmailAddress (element): #common_methods
    '''Checks if element contains e-mail address.
    @-char is identifier.
    Return e-mail address, None if element didn't contain e-mail.
    '''
    if findStringFromParagraphs('@',element):
        return getTextContent(element)
    return None

def getMeta(documentDict):
    return ooo_meta_inspector.getMeta(documentDict)

#TODO: käy läpi office:body --> text:p -> text:style-name="T1"
def checkIllegalStyling (documentDict):
    if getStyleElementByStyleName(documentDict,'T1') is not None:
        return False
    else:
        return True
    
#TODO:     
def checkTextParagraphs (documentDict,expectedStyle):
    textElements = documentDict['content.xml'].getElementsByTagName('text:p')
    for element in textElements:
        styleName = element.getAttribute ("text:style-name")
        if  styleName != expectedStyle:
            parentStyle = getParentStyleList(documentDict,styleName) 
            if parentStyle[1] != expectedStyle: #[0] is style itself, [1] is parent style
                return False
                
        
                


def test(documentDict):  
    '''For testing...
    '''
      

    
    print getListOfUsedStyleElements (documentDict)
    if getStyleElementByStyleName(documentDict,'T1') is not None:
        print 'VIRHE: Dokumentissa on kaytetty tyylien ulkopuolista muotoilua'
    
    sectionBreakElements = getSectionBreakElements ()
    
    print '\nKANSILEHTI:'
    firstElement = sectionBreakElements[0]
    pageStyleName = firstElement.getAttribute ('text:style-name')        
    pageStyleElement = getStyleElementByStyleName (documentDict,pageStyleName)        
    pageMasterPageName = getMasterPageStyleName (pageStyleElement)
    pageMasterPage = getMasterPageStyleElement(documentDict,pageMasterPageName)

    print "Ensimmäinen tekstikappale:",getTextContent(sectionBreakElements[0])
    printPageLayoutProperties (getPageLayoutElement(documentDict,pageMasterPage.getAttribute('style:page-layout-name')))
    checkAuthorAndPageNumber(pageMasterPage)
    
    print "\nSISÄLLYLUETTELO SIVU:"
    firstElement = sectionBreakElements[1]
    pageStyleName = firstElement.getAttribute ('text:style-name')        
    pageStyleElement = getStyleElementByStyleName (documentDict,pageStyleName)        
    pageMasterPageName = getMasterPageStyleName (pageStyleElement)
    pageMasterPage = getMasterPageStyleElement(documentDict,pageMasterPageName)

    print "Ensimmäinen tekstikappale:",getTextContent(sectionBreakElements[1])
    printPageLayoutProperties (getPageLayoutElement(documentDict,pageMasterPage.getAttribute('style:page-layout-name')))
    checkAuthorAndPageNumber(pageMasterPage)
    
                        
    print "\nVARSINAINEN TEKSTI:"
    try:
        firstTextPage = sectionBreakElements[2]
        firstTextPageStyleName = firstTextPage.getAttribute ('text:style-name')
        firstTextPageStyleElement = getStyleElementByStyleName(firstTextPageStyleName)
        firstTextPageMasterPageName = getMasterPageStyleName(firstTextPageStyleElement)
        firstTextPageMasterPage = getMasterPageStyleElement(firstTextPageMasterPageName)
        
        print "Ensimmäinen tekstikappale:",getTextContent(sectionBreakElements[2])
        printPageLayoutProperties (getPageLayoutElement(firstTextPageMasterPage.getAttribute('style:page-layout-name')))
        checkAuthorAndPageNumber(firstTextPageMasterPage)
    except:
        print "ei ole osanvaihtoa"
    
    print ''
    
    findEmptyParagraphs(documentDict)
    print ''
    getTOC (documentDict)
    print''
    getHeadingList(documentDict)
    print ''
#    if findStringFromParagraphs('*'):
#        print "löytyi merkki"
#    else:
#        print "ei löytynyt merkkiä"
    

    print ''
    checkHeadingNumbering(documentDict)
    print ''
    getImagePaths(documentDict)
    print''
    getImageCaption(documentDict)
    print ''
    print 'käytetyt tyylit'
    for styleName in getListOfUsedStyleNames ():
        if styleName is not None:
            print styleName.upper()
            #getStyle(styleName)
            #print ''
    print ''
    checkEndnotesAndFootnotes()
    print''
    print 'META'
    meta =  ooo_meta_inspector.getMeta(documentDict)
    for i in meta.keys():
        print i,":", meta[i]
    print''
   
        

        
    

    
if __name__ == "__main__":
    '''For testing...
    '''
    filename ='sampleFiles/odt/Yliopisto_opiskelu.odt'
    #filename ='sampleFiles/odt/tekstinkasittely_ooo_toc.odt'
    #filename = 'sampleFiles/odt/projektisuunnitelma_0.3.0_ollin_kommentit.odt'
    #filename ='sampleFiles/odt/calc_objektit.odt'
    
    documentDict = makeDict(filename)
    
    #test(documentDict)
    #getCoverPageElements()
    #print getPageMarginals()
    #print getPageSize()
    #checkCoverPageContent()
    #getIndexContent()
#    print checkTable(documentDict)
#    getTableDict (documentDict)
#    print documentDict['content.xml']
#    print documentDict['Object 2/content.xml']
#    for object in getObjectPaths(documentDict):
#        checkObject(documentDict,object.split('/')[1])
#    print checkList(documentDict)
#    printLists(documentDict)
#    print getHeadingList(documentDict)
#    print getTOC(documentDict)
#    #print checkTocContent()
#    #print getStyleDisplayNameByStyleName('tekija_5f_email')
#    print documentDict['content.xml']
#    print documentDict['Object 2/content.xml']
#    print getSectionBreakElements ()
    print getTableDict(documentDict)
    print getHeadingList(documentDict)
    print checkHeadingNumbering(documentDict)
    for i in getStyle(documentDict, "Heading_20_1").items():
        print i[0] ,":",i[1]
    print checkTextParagraphs (documentDict,"Text_20_body")
    print checkHeadingNumbering(documentDict)
    print getParentStyleList(documentDict,"tekija_5f_email")
    print getParentStyleList(documentDict,"Text_20_body")
    print getIndexContentFromDocument(documentDict)
    print getIndexContent (documentDict)
    print checkIndexContent()
    
    

    