#!/usr/bin/python
# -*- coding: UTF-8 -*-

'''
Created on Mar 10, 2011

@author: juiitamm
'''

import zipfile
import word_processing
from cStringIO import StringIO
from xml.dom.minidom import getDOMImplementation

def checkZipFiles(zipFile, fileName, feedback):
    """ Checks if a file is a zip-file.
    Recursively checks if files inside a zip-file are zip-files.
    If the file is a zip-file and it's in Microsoft Office or OpenOffice.org file format, gives the file forward.
    """
    fileExtension = getFileExtension(fileName)
    
    if fileExtension == "zip":
        try:
            zipDocument = zipfile.ZipFile(zipFile)
        except:
            feedback.append(fileName + " is not a zip-file!")
            return
        
        for f_info in zipDocument.infolist():
            data = zipDocument.read(f_info.filename)
    # If you need a file like object, wrap it in a cStringIO
            fobj = StringIO(data)
            checkZipFiles(fobj, f_info.filename, feedback)
            
    elif(fileExtension  == "docx"):
        results = word_processing.inspect(zipFile, "", fileExtension)
        feedback.append(resultsToDom(fileName, results))
    elif(fileExtension  == "pptx"):
        feedback.append("pptx not implemented.")
    elif(fileExtension  == "xlsx"):
        feedback.append("xlsx not implemented.")
    elif(fileExtension  == "odt"):
        results = word_processing.inspect(zipFile, "", fileExtension)
        feedback.append(resultsToDom(fileName, results))
    elif(fileExtension  == "odp"):
        feedback.append("odp not implemented.")
    elif(fileExtension  == "ods"):
        feedback.append("ods not implemented")
    elif(fileExtension  == "docm"):
        feedback.append(data.filename + " is a docm-file, but it should be docx-file.")
        word_processing.inspect(zipFile, "", fileExtension)
    elif(fileExtension  == "pptm"):
        feedback.append(fileName + " is a pptm-file, but it should be pptx-file. Not implemented.")
    elif(fileExtension  == "ppsx"):
        feedback.append(fileName + " is a ppsx-file, but it should be pptx-file. Not implemented.")
    else:
        feedback.append("File " + fileName + " not supported.")
        return
        


def resultsToDom(fileName, results):
    feedbackCount = 0
    
    impl = getDOMImplementation()
    Document = impl.createDocument(None, None, None)
    
    h21 = Document.createElement('h2')
    txt1 = Document.createTextNode(fileName)
    h21.appendChild(txt1)
    
    p1 = Document.createElement('p')
    
    div1 = Document.createElement('div')
#    div1.setAttribute('id','filename');
    for key in results.keys():
        h31 = Document.createElement('h3')
        div1.appendChild(h31)
        txt6 = Document.createTextNode(key.encode('iso-8859-1'))
        h31.appendChild(txt6)
        
        for item in results[key]:
            
            p2 = Document.createElement('p')
            div1.appendChild(p2)
            txt8 = Document.createTextNode(item.encode('iso-8859-1'))
            p2.appendChild(txt8)
            feedbackCount += 1
    summaryText = 'Virheit ' + str(feedbackCount) + ' kappaletta.'
    txt3 = Document.createTextNode(summaryText.decode('iso-8859-1').encode('iso-8859-1'))
    p1.appendChild(txt3)
    
    
    return h21.toxml() + p1.toxml() + div1.toxml()
        
def getFileExtension(fileName):
    ''' Splits the given string at char . and returns what is left on the right side of the last . char.
        Returns empty string if . char is not found.
    '''
    list = fileName.split(".")
    return str.lower(list[len(list) - 1])
            
def beginInspection(sentfile, filename, feedback):
    #if len(sentfile.value) == 0:
    #    feedback.append("File not found!")
    #    return    
    checkZipFiles(sentfile, filename, feedback)
            
if __name__ == '__main__':
    feedback = []
#    )
#    print getFileExtension("testi.odt")
#    print getFileExtension("testi.DOCX")
#    print getFileExtension("testi.ODT")
#    print getFileExtension("testi.zip")
#    print getFileExtension("TESTI.ZIP")
#    print getFileExtension("testi")
#    print getFileExtension("TESTI")
#    
#    #checkZipFiles("harkka.zip")
#    #checkZipFiles("harkka_tuplazip.zip")
#    checkZipFiles("harkka_triplazip.zip")
#    checkZipFiles('sampleFiles/docx/Teija_Holtta.docx')
#    checkZipFiles("vaatimukset.xml")
#    checkZipFiles("foobar")
#    checkZipFiles("requirements.py")
    checkZipFiles("sampleFiles/odt/Yliopisto_opiskelu.odt", "Yliopisto_opiskelu.odt", feedback)
    checkZipFiles("sampleFiles/docx/Teija_Holtta.docx", "Teija_Holtta.docx", feedback)
#    checkZipFiles("sampleFiles/odt/tekstinkasittely_ooo.odt")
#    checkZipFiles("sampleFiles/odt/calc_objektit.odt")

    

    for item in feedback:
        print item
    