#!/usr/bin/python
# -*- coding: UTF-8 -*-

import sys
import conversions
from common_methods import *
import zipfile
import xml.dom.minidom
from requirements import Requirements
import mso_meta_inspector
import string
from xml.dom import minidom
from math import fabs
from conversions import convertTwipToCm, convertTwipToPt
#from word_processing import styleXml

errors = []
pageBreak = 'w:lastRenderedPageBreak'

'''
Trying to generate some valid pydoc comments.
@author Vili Auvinen, Juho Tammela
@version 0.1
'''

def appendError(error):
    errors.append(error)

def getStyleElementById(styleId, styleXml):
    ''' Return style-element with a given style id.'''
    styleElements = getElements('w:style', styleXml)
    for element in styleElements:
        if (element.getAttribute('w:styleId') == styleId):
            return element
    
    errors.append(styleId + ' -style id is not found.')
    return None
            
def getStyleElementByName(styleName, styleXml):
    ''' Return style-element with a given style name. '''
    nameElements = getElements('w:name', styleXml) # w:name -element is style-element's child
    for element in nameElements:
        if (element.getAttribute('w:val') == styleName):
            return element.parentNode
    errors.append(styleName + ' -style name is not found.')
    return None

def getBasedOnStyleName (styleName, styleXml):
    '''Return the name of the based on -style for a given style name.'''
    # FIXME: returns actually the styleId of the basedOn-style NOT the styleName.
    if getStyleElementByName(styleName, styleXml) is not None:
        try:
            return getStyleElementByName(styleName, styleXml).getElementsByTagName('w:basedOn')[0].getAttribute('w:val')
        except:
            return None

def getStyleName(styleElement):
    ''' Return the style name of a given style element.'''
    # TODO: try except
    return styleElement.getElementsByTagName('w:name')[0].getAttribute('w:val')

def getStyleNameById(styleId, styleXml):
    ''' Return the name of a style with a given style id. '''
    styleElement = getStyleElementById(styleId, styleXml)
    if styleElement is not None:
        return getStyleName(styleElement)
    return None

def getStyleIdByName(styleName, styleXml):
    ''' Gets the styleId of the style-element with a given style name.
    
    styleName -- name of the style
    
    return -- styleId
    '''
    styleElements = getElements('w:name', styleXml)
    for styleElement in styleElements:
        if (styleElement.getAttribute('w:val') == styleName):
            styleId = styleElement.parentNode.getAttribute('w:styleId')
            return styleId
            
#    errors.append('Element ' + styleName + ' is missing.')
    return None

def getParagraphStyleId(p):
    try:
        styleId = p.getElementsByTagName('w:pStyle')[0].getAttribute('w:val')
    except:
        return None
    return styleId

#def getParentStyleList (styleName): # not used yet
#    '''Lists styles parent styles.
#       Returns a list of parent styles (lists first entry is style itself).
#    '''
#    parentStyleList = [styleName]
#       
#    while getBasedOnStyleName(styleName):
#        styleName = getBasedOnStyleName(styleName)
#        parentStyleList.append(styleName)
#    
#    return parentStyleList
#
#def checkAttributes(styleAttributeList, element):
#    if element:
#        for attribute in styleAttributeList.keys():
#            if element.hasAttribute(attribute):
#                styleAttributeList[attribute] = element.getAttribute(attribute)
#    
#    return styleAttributeList
#
#def getStyleAttributes (styleAttributeList, styleElementsList):
#   
#    
#    styleElement = getStyleElementByName(styleAttributeList['style'])
#    paragraphPropertiesElement = styleElement.getElementsByTagName('w:pPr')
#    runPropertiesElement = styleElement.getElementsByTagName('w:rPr')
#    
#    if paragraphPropertiesElement:        
#        for elementName in styleElementsList.keys():
#            if (len(paragraphPropertiesElement[0].getElementsByTagName(elementName)) != 0):
#                element = paragraphPropertiesElement[0].getElementsByTagName(elementName)[0]
#                for attribute in styleAttributeList.keys():
#                    if element.hasAttribute (attribute):
#                        styleAttributeList[attribute] = element.getAttribute (attribute)
#                styleElementsList[elementName] = styleAttributeList
#
#    #===========================================================================
#    # if paragraphPropertiesElement:        
#    # for elementName in styleElementsList.keys():
#    #    if (len(paragraphPropertiesElement[0].getElementsByTagName(elementName)) != 0):
#    #        styleElementsList[elementName] = paragraphPropertiesElement[0].getElementsByTagName(elementName)[0]
#    #        for attribute in styleAttributeList.keys():
#    #            if styleElementsList[elementName].hasAttribute (attribute):
#    #                styleAttributeList[attribute] = styleElementsList[elementName].getAttribute (attribute)
#    #===========================================================================
#
#                        
#    print styleElementsList
#    
#    return styleElementsList

def getThemeFont(themeXml, styleDefinitions, themeName):
    if themeName == "" or themeName is None:
        return styleDefinitions
    
    fontElement = None
    
    if themeName.startswith('major'):
        fontElement = themeXml.getElementsByTagName('a:majorFont')[0]
    elif themeName.startswith('minor'):
        fontElement = themeXml.getElementsByTagName('a:minorFont')[0]
        
    if fontElement is not None:
        themeFont = fontElement.getElementsByTagName('a:latin')[0].getAttribute('typeface')
        styleDefinitions['w:ascii'] = themeFont
        styleDefinitions['w:asciiTheme'] = None
    
    return styleDefinitions

def getStyleDefinitions (element, styleDefinitions):
    ''' Return style definitions of a given element.
    First checks if the element has any children and uses recursion if some are found.
    Next checks if the element has attributes.
      * If the attribute name is a key in the dict, stores the value of the attribute.
      * If the attribute name is 'w:val' and the element tag name is a key in the dict, stores the value of the attribute.
    If the element tag name is a key in the dict and the element doesn't have any attributes or children, stores value '1' in the dict.
    
    element -- style definitions are searched inside this element
    styleDefinitions -- a dict where the style definitions are stored. 
                        May contain tag names or attribute names and some default values.
    
    return -- returns the style definitions in a dict.
    '''
    for child in element.childNodes:
        if child.nodeType != child.TEXT_NODE:
            styleDefinitions = getStyleDefinitions(child, styleDefinitions)
    
    if element.hasAttributes():
        for i in range (0, element.attributes.length):
            attributeName = element.attributes.item(i).name
            if styleDefinitions.has_key(attributeName):
                styleDefinitions[attributeName] = element.attributes.item(i).value
#            if attributeName != "w:val":
#                styleDefinitions[element.attributes.item(i).name] = element.attributes.item(i).value
            elif styleDefinitions.has_key(element.tagName) and attributeName == "w:val":
                styleDefinitions[element.tagName] = element.attributes.item(i).value
    
    elif styleDefinitions.has_key(element.tagName) and element.hasChildNodes() is False:
        styleDefinitions[element.tagName] = '1'
        
    return styleDefinitions


def getStyle(document, styleName):
    styleXml = document['word/styles.xml']
    themeXml = document['word/theme/theme1.xml']
    
    styleDefinitions = getCompleteStyleDefinitions(styleXml, styleName, themeXml)
    
    if styleDefinitions is None:
        return False
    
    translateDict = {'w:name':'styleName', 
        'w:ascii':'fontName', 
        'w:sz':'fontSize',
        'w:caps':'transform', 
        'w:left':'indent', 
        'w:line':'linespacing',
        'w:before':'spacingBefore',
        'w:after':'spacingAfter',
        'w:keepNext':'keepWithNext', 
        'w:jc':'alignment',
        'w:widowControl':'widowControl', 
        #'w:widowControl':'widows', #TODO: widows kovakoodattuna
        'w:b':'bold',
        'w:i':'italic'}
    styleDict = {}
    
    # riviväli menee 12 pointin eli 240 twipin kerrannaisissa
    # <w:spacing w:before="300" w:after="340" w:line="240" w:lineRule="auto"/>
    # esim. w:line="360" niin riviväli on 1.5
    styleDefinitions['w:line'] = float(styleDefinitions['w:line'])/float(240.0)
    
    styleDefinitions['w:sz'] = int(styleDefinitions['w:sz']) / 2
    styleDefinitions['w:before'] = convertTwipToPt( float(styleDefinitions['w:before']) )
    styleDefinitions['w:after'] = convertTwipToPt( float(styleDefinitions['w:after']) )
    styleDefinitions['w:left'] = round( convertTwipToCm( float(styleDefinitions['w:left']) ), 1)
    # pyöristetään yhden tarkkuudella, jos käytetään useammin niin päivitä conversions.py
    
    

    for key in translateDict.keys():
        styleDict[translateDict[key]] = styleDefinitions[key]

    printDict(styleDict)
    
    return styleDict

def getCompleteStyleDefinitions(styleXml, styleName, themeXml):
    ''' Return the style definition of a given style.
    Recursion used because a style can be based on some other style.
    In addition, base style gets style definitions from document defaults.
    Finally, some style definions are not found in the xml-file at all. These definitions use some default value which must be assumed.
    
    <w:style w:type="paragraph" w:default="1" w:styleId="Normaali">
    <w:name w:val="Normal"/>
    <w:qFormat/>
    <w:rsid w:val="006B493C"/>
    <w:pPr>
        <w:spacing w:before="140" w:after="220" w:line="360" w:lineRule="auto"/>
        <w:ind w:left="567"/>
        <w:jc w:val="both"/>
    </w:pPr>
    <w:rPr>
        <w:rFonts w:ascii="Georgia" w:hAnsi="Georgia"/>
        <w:lang w:val="fi-FI"/>
    </w:rPr>
    </w:style>
    
    styleXml -- the styles.xml-file as a Dom-tree.
    styleName -- the name of the style (NOT the id).
    
    return -- style definitions in a dict.
    '''
    
#    If fontSize is not found mentioned in xml, the user has used the default size which is 12 (*2 = 24) for text body    
#    TODO: initialize dict with the default style definitions.
#    TODO: some of the values are in twips, convert to cm.
    
#     Complete style-element specification can be found at: http://www.schemacentral.com/sc/ooxml/e-w_style-1.html
#     Style-element content for the most part:
#     w:name [0..1]    Primary Style Name
#     w:aliases [0..1]    Alternate Style Names
#     w:basedOn [0..1]    Parent Style ID
#     w:next [0..1]    Style For Next Paragraph
#     w:link [0..1]    Linked Style Reference
#     w:autoRedefine [0..1]    Automatically Merge User Formatting Into Style Definition
#     w:hidden [0..1]    Hide Style From User Interface
#     w:semiHidden [0..1]    Hide Style From Main User Interface
#     w:unhideWhenUsed [0..1]    Remove Semi-Hidden Property When Style Is Used
#     w:qFormat [0..1]    Primary Style
#     w:locked [0..1]    Style Cannot Be Applied
#    w:pPr: - paragraph properties
#        w:pStyle [0..1]    Referenced Paragraph Style
#         w:keepNext [0..1]    Keep Paragraph With Next Paragraph
#         w:keepLines [0..1]    Keep All Lines On One Page
#         w:pageBreakBefore [0..1]    Start Paragraph on Next Page
#         w:widowControl [0..1]    Allow First/Last Line to Display on a Separate Page
#         w:numPr [0..1]    Numbering Definition Instance Reference
#             w:numPr/w:numId [0..1]    Numbering Definition Instance Reference
#         w:spacing [0..1]    Spacing Between Lines and Above/Below Paragraph
#            w:before    [0..1]    Spacing Above Paragraph    
#            w:beforeLines    [0..1]    Spacing Above Paragraph IN Line Units    
#            w:beforeAutospacing    [0..1]    Automatically Determine Spacing Above Paragraph    
#            w:after    [0..1]    Spacing Below Paragraph    
#            w:afterLines    [0..1]    Spacing Below Paragraph in Line Units    
#            w:afterAutospacing    [0..1]    Automatically Determine Spacing Below Paragraph    
#            w:line    [0..1]    Spacing Between Lines in Paragraph    
#            w:lineRule    [0..1]    Type of Spacing Between Lines
#         w:ind [0..1]    Paragraph Indentation
#            w:left    [0..1]    Left Indentation    
#            w:leftChars    [0..1]    Left Indentation in Character Units    
#            w:right    [0..1]   Right Indentation    
#            w:rightChars    [0..1]    Right Indentation in Character Units    
#            w:hanging    [0..1]    Indentation Removed from First Line    
#            w:hangingChars    [0..1] Indentation Removed From First Line in Character Units    
#            w:firstLine    [0..1] Additional First Line Indentation    
#            w:firstLineChars    [0..1] Additional First Line Indentation in Character Units
#         w:jc [0..1]    Paragraph Alignment
#         w:outlineLvl [0..1]    Associated Outline Level
#    w:rPr - run properties
#        2. w:rFonts [0..1]    Run Fonts
#                w:ascii    [0..1]    w:ST_String    ASCII Font    
#                w:hAnsi    [0..1]    w:ST_String    High ANSI Font
#                w:cs    [0..1]    w:ST_String    Complex Script Font    
#                w:asciiTheme    [0..1]    w:ST_Theme    ASCII Theme Font    
#                w:hAnsiTheme    [0..1]    w:ST_Theme    High ANSI Theme Font
#                w:cstheme    [0..1]    w:ST_Theme    Complex Script Theme Font
#         3. w:b [0..1]    Bold
#         4. w:bCs [0..1]    Complex Script Bold
#         5. w:i [0..1]    Italics
#         6. w:iCs [0..1]    Complex Script Italics
#         7. w:caps [0..1]    Display All Characters As Capital Letters
#         8. w:smallCaps [0..1]    Small Caps
#         9. w:strike [0..1]    Single Strikethrough
#        10. w:dstrike [0..1]    Double Strikethrough
#        11. w:outline [0..1]    Display Character Outline
#        15. w:noProof [0..1]    Do Not Check Spelling or Grammar
#        24. w:sz [0..1]    Font Size
#        25. w:szCs [0..1]    Complex Script Font Size
#        27. w:u [0..1]    Underline
#        36. w:lang [0..1]    Languages for Run Content
            
#     Initialize the styleDefinitions dict with the style definitions you wish to search and their default values.
#     Use attribute name as a key.
#     If attribute name is 'w:val', use element tag name as a key.
#     If the element can be empty and attributes are optional, use element tag name as a key.
    styleDefinitions = {'w:name': None, 
                       'w:basedOn': None,
                       'w:next': None,
                       'w:link': None,
                       'w:autoRedefine': None,
                       'w:left': '0',
                       'w:right': '0',
                       'w:firstLine': None,
                       'w:line': '240.0',
                       'w:before': None,
                       'w:after': None,
                       'w:widowControl': '1', # oletuksena päällä
                       'w:jc': None,
                       'w:sz': '24',
                       'w:ascii': None,
                       'w:asciiTheme': None,
                       'w:b': None,
                       'w:i': None,
                       'w:u': None,
                       #'w:outline': None,
                       #'w:numId': None,
                       #'w:ilvl': None,
                       'w:lang': None,
                       'w:keepNext': None,
                       'w:keepLines': None,
                       'w:pageBreakBefore': None,
                       'w:caps': None}
    
#    styleDefinitions = dict()

    styleElement = getStyleElementByName(styleName, styleXml)
    
    if styleElement is None:
        return
        
    basedOnStyleId = getBasedOnStyleName(styleName, styleXml)

#     Recursion: if this style has a basedOn-style, method calls itself with the basedOnStyleName. 
#     When style has no basedOn-style, get the definitions in w:docDefaults-element first.
    if basedOnStyleId is not None:
        styleDefinitions = getCompleteStyleDefinitions(styleXml, getStyleNameById(basedOnStyleId, styleXml), themeXml)
    elif basedOnStyleId is None:
        styleDefinitions = getStyleDefinitions(styleXml.getElementsByTagName('w:docDefaults')[0], styleDefinitions)
    
    styleDefinitions = getStyleDefinitions(styleElement, styleDefinitions)
    
    if(styleDefinitions.has_key('w:asciiTheme')):
        styleDefinitions = getThemeFont(themeXml, styleDefinitions, styleDefinitions['w:asciiTheme'])
        
#    translatedStyleDefinitions = {}
#    
#    for key in translateDict.keys():
#        translatedStyleDefinitions[translateDict[key]] = styleDefinitions[key]
        
#    return translatedStyleDefinitions
    
    return styleDefinitions

def getElementValue(elementTagName, xmlData):
    ''' Gets the element value of a certain tag from the given xml file. '''
    if (xmlData):
        try:
            value = xmlData.getElementsByTagName(elementTagName)[0].firstChild.nodeValue
        except:
            return None
        return value
    
def getElementValueWithinElement(element, elementTagName):
    ''' Returns a certain element value of a the given parent element by the given elementTagName. 
    
    element -- the element where the search begins
    elementTagName -- the elements that are looked for from element'''
    try:
        value = element.getElementsByTagName(elementTagName)[0].firstChild.nodeValue
    except IndexError:
        return None
        #return elementTagName + " not found in " + element
    return value

def getElementWithinElement(element, elementTagName):
    ''' Returns a certain element of a the given parent element by the given elementTagName. 
    
    element -- the element where the search begins
    elementTagName -- the elements that are looked for from element'''
    try:
        value = element.getElementsByTagName(elementTagName)[0]
    except IndexError:
        return None
    return value

def getElementsWithinElement(element, elementTagName):
    ''' Returns a certain element of a the given parent element by the given elementTagName. 
    
    element -- the element where the search begins
    elementTagName -- the elements that are looked for from element
    
    return -- elements in a list
    '''
    elements = element.getElementsByTagName(elementTagName)
    if len(elements) == 0: return None

    return elements

def getElements(elementTagName, xmlData):
    ''' Returns the elements of the given xml file if xmlData and the given elementTagName 
        exist in the xmlData. 
       
       xmlData - the given xml file'''
    if (xmlData):
        elements = xmlData.getElementsByTagName(elementTagName)
        if len(elements) == 0: return None
    else: return xmlData + ' is not working.'
    
    return elements


def getTargetXmlFileByHeader(header): #TODO: try except
    targetFile = getRelsTargetByRId(header.getAttribute('r:id'), docRelsXml)
    targetFile = zip.read('word/' + targetFile) # omaan metodiin
    targetFileXml = minidom.parseString(targetFile)
    return targetFileXml

def checkFrontPageHeadersAndFooters(references):
    ''' Checks if there are headers or footers in the front page by looking for <w:t> tags (could be better)
        and appends an error if there are.
        
        references -- header or footer references
    '''
    if references is not None:
        for header in references:
            targetFileXml = getTargetXmlFileByHeader(header)
            if getElements('w:t', targetFileXml) is not None: #TODO: eksaktimmin, getElementValue?
                errors.append('Kansisivulla ei saa olla ylä- tai alatunnisteita')
                
def checkAutomaticPageNumbering(references):
    '''
        Checks if header or footer has a <w:instrText> tag with the content of PAGE \* MERGEFORMAT which 
        means it is automatically generated.
        
        references -- header or footer references
        
        return -- True if page numbering is automatically made, False if not
    '''
    if references is not None:
        for header in references:
            targetFileXml = getTargetXmlFileByHeader(header)
            elementValue = getElementValue('w:instrText', targetFileXml)
            if elementValue is None: return False
            if elementValue.find('PAGE') and elementValue.find('MERGEFORMAT'): # \ vai \\, find('PAGE \* MERGEFORMAT) ei toiminut
                return True
    return False
    
def checkNameInHeaderOrFooter(references):
    if references is not None:
        for header in references:
            targetFileXml = getTargetXmlFileByHeader(header)
            pElement = getElements('w:p', targetFileXml)[0] # improve
            textContent = getTextContent(pElement)
            if textContent.find('Vili Auvinen'): return True #getCreator() lopulta tähän
        return False


def checkNumFormatAndStartNumber(sectionPgNumType):
    ''' Checks the number format and start number of the given section page number type.
        
        sectionPgNumType -- the given page number type of a certain section
        
        return -- the numbering format
        ''' 
    numFormat = 'Standard'
    if sectionPgNumType:
        if sectionPgNumType.hasAttribute('w:fmt'):
            numFormat = sectionPgNumType.getAttribute('w:fmt')
        if not sectionPgNumType.getAttribute('w:start') == '1':
            errors.append('Varsinaisen dokumenttiosan numerointi on aloitettava numerosta yksi.')
    return numFormat

def checkHeadersAndFooters():
    ''' Kansisivulla (allSectionProperties[0]) ei saa olla header ja footer -elementtejä
        sisällysluettelolla (allSectionProperties[1]) pitää olla eri numerointi (pgNumType.hasAttribute('w:fmt')
        (<w:pgNumType w:fmt="lowerRoman" w:start="1"/>) kuin viimeisellä,
        löydyttävä fmt, muualla oltava normaali eli <w:pgNumType w:start="1"/>
        
        Entäs jos sektioita on dokumentissa enemmän kuin kolme? Sittenpähän on.
    '''
    
    allSectionProperties = getElements('w:sectPr', doc)
    
    if len(allSectionProperties) < 3: #TODO: tarvitseeko? rakenne pitää tsekata erikseen ennen muita
        errors.append('There must be at least three section breaks in the whole document.')
        return
    #allSectionProperties[0] = kansisivu
    #allSectionProperties[1] = sisällysluettelo
    #allSectionProperties[2] = loput dokumentista
    
    tocSectionPr = allSectionProperties[1] # näiden välissä saa olla vain sisällysluettelotyylejä? footerit ja headerit?
    commonSectionPr = allSectionProperties[2]
    
    frontPageHeaderReferences = getElementsWithinElement(allSectionProperties[0], 'w:headerReference')
    frontPageFooterReferences = getElementsWithinElement(allSectionProperties[0], 'w:footerReference')
    
    checkFrontPageHeadersAndFooters(frontPageHeaderReferences)
    checkFrontPageHeadersAndFooters(frontPageFooterReferences)
    
    tocHdrRefs = getElementsWithinElement(allSectionProperties[1], 'w:headerReference')
    tocFtrRefs = getElementsWithinElement(allSectionProperties[1], 'w:footerReference')
    
    if checkAutomaticPageNumbering(tocHdrRefs) is False and \
    checkAutomaticPageNumbering(tocFtrRefs) is False:
        errors.append("Numerointia ei ole tehty (automaattisesti) oikein sisällysluettelosivulla.")
  
    tocSectionPgNumType = getElementWithinElement(tocSectionPr, 'w:pgNumType')
    
    tocNumFormat = checkNumFormatAndStartNumber(tocSectionPgNumType)
    commonSectionPrPgNumType = getElementWithinElement(commonSectionPr, 'w:pgNumType')
    
    commonNumFormat = checkNumFormatAndStartNumber(commonSectionPrPgNumType)
    
    if commonNumFormat == tocNumFormat:
        errors.append('Sisällysluettelosivulla on oltava erilainen ja automaattisesti generoitu numerointityyli kuin muualla.')
    
    commonHdrRefs = getElementsWithinElement(allSectionProperties[2], 'w:headerReference')
    commonFtrRefs = getElementsWithinElement(allSectionProperties[2], 'w:footerReference')    
    
    # löytyykö dokumentin laatijan nimi ylä- tai alatunnisteesta ja löytyykö MERGEFORMATTEJA
    if checkNameInHeaderOrFooter(commonHdrRefs) is False and \
    checkNameInHeaderOrFooter(commonFtrRefs) is False:
        errors.append("Nimeä ei löydy dokumentin ylä- tai alatunnisteista.")
    
    if checkAutomaticPageNumbering(commonHdrRefs) is False and \
    checkAutomaticPageNumbering(commonFtrRefs) is False:
        errors.append("Sivunumerointia ei ole tehty automaattisesti varsinaisessa dokumentissa.")


def getSectionProperties():
    ''' Checks all the sectPr-tags. There must be at least three section breaks  (first page, toc-page and content) in the whole document. 
        The margins should be the same throughout the whole document?
        
        return -- section properties in a dict'''
    
    finalSectionProperties = dict([['w:w', None], ['w:h', None], ['w:top', None], ['w:right', None], ['w:bottom', None], \
                               ['w:left', None], ['w:right', None], ['w:header', None], ['w:footer', None], \
                               ['w:gutter', None], ['w:start', None], ['w:space', None], ['w:linePitch', None], ['w:titlePg', None]])
    
    allSectionProperties = getElements('w:sectPr', doc) # returns a nodeList
    if len(allSectionProperties) < 3: 
        errors.append('There must be at least three section breaks in the whole document.')
        return
    
    for element in allSectionProperties:
        finalSectionProperties = getStyleDefinitions(element, finalSectionProperties)
        #TODO: conversions from twips to cm

    if finalSectionProperties['w:titlePg'] == '1':
        errors.append('First page must be done with section breaks. You can not use the different first page -checkbox.')
        
    return finalSectionProperties


def setPageProperties(allSectionProperties, pageProperties, tagName):
    for element in allSectionProperties:
        for key in pageProperties.keys():
            size = getElementWithinElement(element, tagName).getAttribute('w:' + key)
            if pageProperties[key] is None:
                pageProperties[key] = size
            elif pageProperties[key] != size:
                pageProperties[key] = False # False tarkoittaa, että tietoa on muutettu
    
    return pageProperties

def convertSizes(dict):
    for key in dict.keys():
        value = dict[key]
        value = convertTwipToCm(int(value))
        roundedValue = round(value, 1) # pyöristetään yhden desimaalin tarkkuudella
        dict[key] = roundedValue
        
    return dict

def getPageMarginals(document):
    allSectionProperties = getElements('w:sectPr', document['word/document.xml'])
    
    pageMarginals = {'top': None, 'right': None, 'bottom': None, 
                     'left': None, 'header': None, 'footer': None}
    
    setPageProperties(allSectionProperties, pageMarginals, 'w:pgMar')            
    
    return convertSizes(pageMarginals)

def getPageSize(document):
    allSectionProperties = getElements('w:sectPr', document['word/document.xml'])
    
    pageSize = {'w': None, 'h': None} # width ja heigth
    
    setPageProperties(allSectionProperties, pageSize, 'w:pgSz')
    
    finalPageSize = {'width': pageSize['w'], 'height':pageSize['h']} 
    
    return convertSizes(finalPageSize)
        
    
def getTitle(coreXml):
    return getElementValue('dc:title', coreXml)

def getCreator(coreXml):
#    return mso_meta_inspector.getCreator(coreXml)
    return getElementValue('dc:creator', coreXml)

def getLastModifier():
    return getElementValue('dcterms:created', coreXml)

def getCreateDate():
    return getElementValue('dcterms:created', coreXml)

def getLastModifiedDate():
    return getElementValue('dcterms:modified', coreXml)
    
def getRevision():
    return getElementValue('cp:revision', coreXml)

    


def getTextFromParagraph(paragraph):
    '''
    Gets the text content of <w:t>-elements from the given <w:p> paragraph.
    
    return -- text values in a list
    '''
    #textList = []
    eventualText = ''
    # gets all text elements from the given paragraphs
    #for element in paragraphs:
    textElements = getElementsWithinElement(paragraph, 'w:t')
    if textElements is not None:
        for i in textElements:
            eventualText += i.firstChild.nodeValue
        #textList.append(eventualText)
        #eventualText = ''
    return eventualText
    #return textList

def checkTocContent(document):
    ''' Checks if all of the headings created in the document are listed in the table of contents. 
        If not, an error is appended.
    '''
    
    docXml = document['word/document.xml']
    styleXml = document['word/styles.xml']
    
    headingParagraphs = getParagraphElementsBySequentialStyleName("heading", styleXml , docXml)
    #docHeadings = getTextFromParagraph(headingParagraphs)
        
    docTocStyles = getParagraphElementsBySequentialStyleName('toc', styleXml, docXml)
    #tocHeadings = getTextFromParagraph(docTocStyles)

    docHeadings = []
    for heading in headingParagraphs:
        docHeadings.append(getTextFromParagraph(heading).strip())
    
        
    tocHeadings = []
    for tocStyle in docTocStyles:
        tocHeadings.append(getTextFromParagraph(tocStyle).strip())
    
    docHeadingsLength = len(docHeadings)
    tocHeadingsLength = len(tocHeadings)
    listLength = len(docHeadings) # oletuksena docHeadingsien pituus
    
    if docHeadingsLength != tocHeadingsLength:
        return False
        # errors.append('Sisällysluettelon otsikkoja on eri määrä kuin dokumentin otsikkoja.')
        #------------------------ # if-else shorthand: x = z if condition else y
        # if docHeadingsLength > tocHeadingsLength: listLength = tocHeadingsLength
        #--------------------------------- else: listLength = docHeadingsLength
        
    i=0
    while i<listLength:
        if string.find (tocHeadings[i], docHeadings[i]) == -1:
            return False
            #errors.append(docHeadings[i] + " -otsikkoa ei löytynyt sisällysluettelosta. Päivitä sisällysluettelo.")
        i += 1
            
    bookmarkStarts = getElements('w:bookmarkStart', docXml)
    instrElements = getElements('w:instrText', docXml)
    for element in docTocStyles:
        instrElements = getElementsWithinElement(element, 'w:instrText')
        for instrElement in instrElements:
            instrElementValue = instrElement.firstChild.nodeValue
            if instrElementValue.find('PAGEREF') != -1: # We only want to handle the tags with value including PAGEREF.
                for bookmark in bookmarkStarts:
                    bookmarkNameValue = bookmark.getAttribute('w:name')
                    if bookmarkNameValue.find(instrElementValue) != -1: # if the same code is in bookmarkStart
                        return False
    
    return True
    
def checkTOC(document):
    ''' Check if table of contents is done correctly. It has to have a page break before (and after) it.
        @see: checkTocContent -- calls the method if there's a table of contents to be found.
       <w:p w:rsidR="004A16ED" w:rsidRDefault="004A16ED" w:rsidP="006158B0">
           <w:pPr>
               <w:pStyle w:val="Otsikko"/>
           </w:pPr>
           <w:r w:rsidRPr="006158B0">
               <w:lastRenderedPageBreak/>
               <w:t>SISALLYSLUETTELO</w:t>
           </w:r>
       </w:p>
       <w:p w:rsidR="002274FC" w:rsidRDefault="00FA6E61">
           <w:pPr>
               <w:pStyle w:val="Sisluet1"/>
       ...
    '''
    
    styleId = getStyleIdByName("toc 1", document['word/styles.xml']) # Sisluet1
    #if (styleId == None): errors.append('There is no table of contents.')
    pStyles = getElements('w:pStyle', document['word/document.xml'])
    for style in pStyles:
        if (style.getAttribute('w:val') == styleId):
            return True
        
            #checkTocContent() # tarkistaa onko sisällysluettelo päivitetty
            
            
#            try: #TODO: should a page break after the table of contents as well, what about section break?
#                #TODO: it's about section break, this code was made for page break. 
#                if (pageBreak == style.parentNode.parentNode.previousSibling.getElementsByTagName('w:r')[0].firstChild.tagName):
#                    print "There is a page break before table of contents."
#                    return
#            except IndexError:
#                errors.append("There is no page break before the table of contents.")
#                return
            
    return False    
    #if not tocExists: errors.append("There is no table of contents at all.")
    
            
    #return styleId    
        
def checkCoverPage(document):
    ''' Checks if the front page is done correctly
    
    return coverPageText -- dictionary with True or False values
    '''
    
    # if rakenne on oikein, do -- rakenteen tarkastus lisättävä alkuun
    
    coverPageText = { 'email': False,
                      'name': False,
                      'title': False }
    
    docXml = document['word/document.xml']
    coreXml = document['docProps/core.xml']

    paragraphs = getElements('w:p', docXml)
    allSectionProperties = getElements('w:sectPr', docXml)
    
    firstPageText = ''
    lastParagraphOfFirstPage = allSectionProperties[0].parentNode.parentNode
    
    # Saves the content of the first page, getting text from the beginning and
    # breaks the loop when the sectPr-node appears.
    # Pitää katsoa, ettei etusivulla ole sisällysluetteloa ym., koska jos section-breakit on väärin, 
    # "etusivun" tietoihin voi valua sisällysluettelo.
    for element in paragraphs:
        firstPageText += getTextContent(element)
        if element.isSameNode(lastParagraphOfFirstPage):
            break

    # title pitää myös löytyä täältä
    if firstPageText.find('@') != -1: # sähköpostiosoite tulee lopuksi käyttöliittymästä 
        coverPageText['email'] = True
    if firstPageText.find(getCreator(coreXml)) != -1:
        coverPageText['name'] = True
    title = getTitle(coreXml)
    if title is not None:
        if firstPageText.find(title) != -1:
            coverPageText['title'] = True

    return coverPageText
    
def getRelsTargetByRId(rId, rels):
    '''Return the value of the Target-attribute of a Relationship-element with the given id in a given rels-file.
    The value of the Target-attribute can be for example a relative path to local xml-files or images. It can also be a hyperlink. 
    
    rId -- Id-attribute value of a Relationship-element.
    rels -- rels-file as a Dom-tree.
    
    return -- value of the Target-attribute if found.
    '''
    for relationship in rels.getElementsByTagName('Relationship'):
        if rId == relationship.getAttribute('Id'):
            return relationship.getAttribute('Target')
    
def getParentParagraph(element, tag = 'w:p'):
    ''' Return the parent <w:p>-element of a given element if there is one.'''

    parent = element.parentNode
    
    while parent is not None:
        try:
            if parent.tagName == tag:
                return parent
            else:
                parent = parent.parentNode
        except AttributeError:
            return None
    return None
        
#    picElements = doc.getElementsByTagName(elementTagName)
#    picParagraphs = []
#    
#    if len(picElements) == 0:
#        errors.append('Pictures not found.')
#        return
#    
#    for picElement in picElements:
#        picParent = picElement.parentNode
#        while picParent is not None:
#            if picParent.tagName == 'w:p':
#                picParagraphs.append(picParent)
#                break
#            else:
#                picParent = picParent.parentNode
#    return picParagraphs
#    
def checkImages(document):
    ''' Check if there is an image in the document.
    
    document -- the document's xml-files' Dom-trees in a dict.
    '''
    #FIXME: what is the difference between pic:pic and w:pict?
    #w:pict is used when pasting a chart from powerpoint or excel?
    picElements = document['word/document.xml'].getElementsByTagName('pic:pic')
    pictElements = document['word/document.xml'].getElementsByTagName('w:pict')
    
    if len(picElements) > 0:
        #errors.append("Pictures not found.")
        return True
    if len(pictElements) > 0:
        return True
    return False
    
def getPicturePaths(rels):
    targets = []
    picElements = doc.getElementsByTagName('pic:pic')
    for picElement in picElements:
        picRId = picElement.getElementsByTagName('a:blip')[0].getAttribute('r:embed')
        targets.append(getRelsTargetByRId(picRId, rels))
    return targets
    
def checkPictureCaptions():
    '''Return True of False depending on if the next paragraph after a picture paragraph uses the caption-style.
    Goes through all picture paragraphs.
    '''
    #FIXME
    picParagraphs = []
    picElements = doc.getElementsByTagName('pic:pic')
    
    for pic in picElements:
        picParagraphs.append(getParentParagraph(pic))
    
    if picParagraphs is None:
        return
# TODO: try-except    
    for p in picParagraphs:
        captionParagraph = p.nextSibling
        try:
            captionParagraphPpr = captionParagraph.getElementsByTagName('w:pPr')[0]
            captionParagraphStyleID = captionParagraphPpr.getElementsByTagName('w:pStyle')[0].getAttribute('w:val')
            captionParagraphStyleName = getStyleNameById(captionParagraphStyleID)
            if captionParagraphStyleName != "caption":
                return False
        except:
            return False
    return True

def checkStyleUsage():
    '''Goes through all paragraph-elements in a document looking for <w:pStyle>-elements.
    
    '''
    
    paragraphs = doc.getElementsByTagName('w:p')
    
    for p in paragraphs:
        styleDefinitions = {'w:autoRedefine': None,
                       'w:left': None,
                       'w:right': None,
                       'w:firstLine': None,
                       'w:line': None,
                       'w:before': None,
                       'w:after': None,
                       'w:widowControl': None,
                       'w:jc': None,
                       'w:sz': None,
                       'w:ascii': None,
                       'w:asciiTheme': None,
                       'w:b': None,
                       'w:i': None,
                       'w:u': None,
                       #'w:outline': None,
                       #'w:numId': None,
                       #'w:ilvl': None,
                       'w:lang': None,
                       'w:keepNext': None,
                       'w:keepLines': None,
                       'w:pageBreakBefore': None}
        
        try:
            style = p.getElementsByTagName('w:pStyle')[0].getAttribute('w:val')
            style = getStyleNameById(style)
#            if style != "":
#                print style
      
# TODO: check the empty paragraphs content to prevent false positives, are they empty text paragraphs or maybe a picture paragraph etc?
# Now just leaves the paragraphs with no text be and doesn't give an error.

        except:
            pContent = getTextContent(p)
            if pContent.strip() != "":
                errors.append("No style used in paragraph: " + str(pContent[:25]))
        try:
            for paragraphProperties in p.getElementsByTagName('w:pPr'):
                for propertyElement in paragraphProperties.childNodes:
# We don't want section properties to be mixed up as manually made style definitions.
                    if propertyElement.tagName != "w:sectPr":
                        styleDefinitions = getStyleDefinitions(propertyElement, styleDefinitions)
            for runProperties in p.getElementsByTagName('w:rPr'):
                styleDefinitions = getStyleDefinitions(runProperties, styleDefinitions)
                
            for key in styleDefinitions.keys():
                if styleDefinitions[key] is not None:
#                    print key + " " + styleDefinitions[key]
                    errors.append("Manual style definition " + key + " in paragraph: " + getTextContent(p))
        except:
            errors.append(sys.exc_info())
            
def checkEndnotesAndFootnotes(document):
    docXml = document["word/document.xml"]
    
    endnotes = docXml.getElementsByTagName('w:endnoteReference')
    footnotes = docXml.getElementsByTagName('w:footnoteReference')

    if len(endnotes) != 0: return True
    if len(footnotes) != 0: return True
    
    # TODO: doublecheck: find the endnote with the id in endnotes.xml. That's where the endnote text is located.
    # TODO: doublecheck: find the footnote in footnotes.xml
    
    return False
    
    
def printDict(dict):
    ''' Prints the given dict in a readable form. Prints all the key-value pairs unless some value is None. '''
    if dict is None:
        return
    
    for item in dict.keys():
        if (dict[item] is not None):
            print item + ": " + str(dict[item])
        
def checkCrossRefrenceToPictureCaption():
    ''' Goes through pictures' captions looking for a reference, then checks if the caption is referenced somewhere.
    
    return -- true if a cross reference is found, false otherwise
    '''
    picParagraphs = []
    for picElement in doc.getElementsByTagName('pic:pic'):
        picParagraph = getParentParagraph(picElement)
        if picParagraph is not None:
            picParagraphs.append(picParagraph)
    
    for p in picParagraphs:
        captionParagraph = p.nextSibling
        try:
            bookmarkStartElement = captionParagraph.getElementsByTagName('w:bookmarkStart')[0]
        except:
            errors.append('Picture\'s caption reference not found.')
            return False
        reference =  bookmarkStartElement.getAttribute('w:name')
        
        for element in doc.getElementsByTagName('w:instrText'):
            elementText = getTextContent(element)
            if elementText.find(reference) != -1:
                return True
    errors.append('Reference to picture caption not found.')
    return False

def getElementByAttributeValue(nodeList, attributeName, attributeValue):
    for element in nodeList:
        if element.getAttribute(attributeName) == attributeValue:
            return element
    return None

def checkHeadingUsage(document):
    '''Check if heading styles are used in the document.
    
    return -- True if heading styles are used, False otherwise.
    '''
    headingParagraphs = document["word/document.xml"].getParagraphElementsBySequentialStyleName("heading", document["word/styles.xml"])
    if len(headingParagraphs) == 0:
        #errors.append("No heading styles used in this document!")
        return False
    return True

def checkHeadingNumbering(document, errorIdsAndPositions):
    ''' Goes through the heading styles used in the document checking that they use a multilevel numbering and
    the numbering is done correctly using styles and that the numbering is connected to other heading styles.
    
    Gets all the heading styles used in the document.
    Searches for the heading's numbering definition reference in styles.xml.
    Next searches the associated numbering definition in numbering.xml.
    Next searches the correct numbering level definition associated to the heading.
    Checks that the numbering is multilevel and done correctly using the heading styles.
    
    styles.xml:
        <w:style w:type="paragraph" w:styleId="Heading2"> - Heading 2 style definition
            <w:name w:val="heading 2"/>
            −
            <w:pPr>
                −
                <w:numPr>
                    <w:ilvl w:val="1"/> - Numbering Level Reference
                    <w:numId w:val="1"/> - Numbering Definition Instance Reference
                </w:numPr>
                <w:outlineLvl w:val="1"/>
            </w:pPr>
        </w:style>
    
    numbering.xml:
        <w:abstractNum w:abstractNumId="0"> - Abstract Numbering Definition
            <w:multiLevelType w:val="multilevel"/> - Abstract Numbering Definition Type
            −
            <w:lvl w:ilvl="0"> - </w:lvl> - Numbering Level Definition
            
            <w:lvl w:ilvl="1"> - Numbering Level Definition
                <w:start w:val="1"/> - Starting Value
                <w:numFmt w:val="decimal"/> - Numbering Format
                <w:pStyle w:val="Heading2"/> - Paragraph Style's Associated Numbering Level
                <w:lvlText w:val="%1.%2"/> - Numbering Level Text
                <w:lvlJc w:val="left"/> - Justification
                −
                <w:pPr> - Numbering Level Associated Paragraph Properties
                    <w:ind w:left="576" w:hanging="576"/>
                </w:pPr>
            </w:lvl>
        </w:abstractNum>
        −
        <w:num w:numId="1"> - Numbering Definition Instance
            <w:abstractNumId w:val="0"/> - Abstract Numbering Definition Reference
        </w:num>
    '''
    
    #errorIdsAndPositions = {'manualNumbering': None}
    docXml = document["word/document.xml"]
    styleXml = document["word/styles.xml"]
   
    try:
        numXml = document['word/numbering.xml']
        #numFile = zip.read('word/numbering.xml')
        #numXml = xml.dom.minidom.parseString(numFile)
    except:
        #errors.append("No heading numbering used at all.")
        return False
   
    
    headingParagraphs = getParagraphElementsBySequentialStyleName("heading", styleXml, docXml)
    #@see: checkHeadingUsage
    #if len(headingParagraphs) == 0:
    #    errors.append("No heading styles used in this document!")
    #    return
    
    usedHeadingsStyleIds = []
    previousHeadingLevel = 0
    for heading in headingParagraphs:
        styleId = heading.getElementsByTagName('w:pStyle')[0].getAttribute('w:val')
        if len(heading.getElementsByTagName('w:ilvl')) > 0 or \
           len(heading.getElementsByTagName('w:numId')) > 0:
            errorIdsAndPositions['manualNumbering'] = getTextContent(heading)
            # errors.append("Manual numbering definitions made in heading: " + getTextContent(heading))
        if usedHeadingsStyleIds.count(styleId) == 0:
            usedHeadingsStyleIds.append(str(styleId))
        headingLevel = int(styleId[len(styleId) - 1])
        if fabs(headingLevel - previousHeadingLevel) > 1:
            errors.append("Otsikoita ei ole käytetty oikealla tavalla peräkkäin.")
        previousHeadingLevel = headingLevel
    
    #Sort the list: ['Heading1', 'Heading2', 'Heading3', ...]
    usedHeadingsStyleIds.sort(cmp=None, key=None, reverse=False)
    #print usedHeadingsStyleIds
    
    previousNumId = None
    
    for headingStyleId in usedHeadingsStyleIds:

        headingLevel = int(headingStyleId[len(usedHeadingsStyleIds[0]) - 1])

        headingStyleElement = getStyleElementById(headingStyleId, styleXml)
        
        # Get the numbering definitions of the heading style.
        # Default ilvl value to 0 -> ilvl-element not found (level is 0).
        styleDefinitions = dict([['w:ilvl', '0'], ['w:numId', None], ['w:outlineLvl', None]])
        styleDefinitions = getStyleDefinitions(headingStyleElement, styleDefinitions)
        
        # Chekc that the numbering style definitions are OK.
        if styleDefinitions['w:numId'] is None:
            errorIdsAndPositions['styleNotUsed'] = headingStyleId
            #errors.append(headingStyleId + " numbering is not used.")
            return
        if previousNumId is not None and styleDefinitions['w:numId'] != previousNumId:
            errorIdsAndPositions['differentNumbering'] = headingStyleId
            #errors.append(headingStyleId + " is using different numbering as the previous level heading style.")
        previousNumId = styleDefinitions['w:numId']
        if int(styleDefinitions['w:ilvl']) != headingLevel - 1:
            errorIdsAndPositions['notMultilevel'] = headingStyleId
            #errors.append(headingStyleId + " numbering level is not correct, numbering is not multilevel.")
        if int(styleDefinitions['w:outlineLvl']) != headingLevel - 1:
            errorIdsAndPositions['outlineLvl'] = headingStyleId
            #errors.append(headingStyleId + " outline level is not correct.")
        
        # Find the numbering definition element associated to the heading style.
        # Get the abstract numbering definition id from the numbering definition element.
        # Find the abstract numbering definition element with the correct id.
        # Find the numbering level definition with the same level that the heading style.
        try:
            numElement = getElementByAttributeValue(numXml.getElementsByTagName('w:num'), 'w:numId', styleDefinitions['w:numId'])
            abstractNumId = numElement.getElementsByTagName('w:abstractNumId')[0].getAttribute('w:val')
            absNumElement = getElementByAttributeValue(numXml.getElementsByTagName('w:abstractNum'), 'w:abstractNumId', abstractNumId)
            lvlElement = getElementByAttributeValue(absNumElement.getElementsByTagName('w:lvl'), 'w:ilvl', styleDefinitions['w:ilvl'])
        except:
#            errors.append(headingName + " numbering level definitions not found.")
            return
        
        # Get the numbering level definitions.
        numDefinitions = dict([['w:start', None], ['w:numFmt', None], ['w:pStyle', None], ['w:lvlText', None], ['w:lvlJc', None], ['w:tentative', None]])
        numDefinitions = getStyleDefinitions(lvlElement, numDefinitions)
        
# TODO: should we check that the numbering is in format 1, 1.1, 1.1.1 etc ?
        if numDefinitions['w:start'] != '1':
            errorIdsAndPositions['numStart'] = headingStyleId
            #errors.append(headingStyleId + " numbering doesn't start at number 1.")
            # <text>Seuraava otsikkotyyli ei aloita numerointia ykkösestä:</text>
        if numDefinitions['w:pStyle'] != headingStyleId:
            errorIdsAndPositions['numberingWrong'] = headingStyleId
            #errors.append(headingStyleId + " numbering is not done correctly using heading styles.")
        if numDefinitions['w:numFmt'] != "decimal":
            errorIdsAndPositions['numFormat'] = headingStyleId
            #errors.append(headingStyleId + " numbering format is not a decimal number.")
    
    return True
    
def getParagraphElementsByStyleId(docXml, styleId):
    ''' Gets all paragraph-elements in the document by a style id.'''
    paragraphList = []
    
    for p in docXml.getElementsByTagName('w:p'):
        try:
            if styleId == p.getElementsByTagName('w:pStyle')[0].getAttribute('w:val'):
                paragraphList.append(p)
        except:
            continue
            #errors.append("No style used in paragraph: " + getTextContent(p))
    return paragraphList
    
def getParagraphElementsBySequentialStyleName(styleNamePrefix, styleXml, docXml):
    ''' Return all paragraph elements that use a style with a sequential numbering.
    For example, gets all paragraphs that use styles with stylenames heading 1, heading 2, etc or
    index 1, index 2, etc.
    
    styleNamePrefix -- the prefix of the sequential style name.
    '''
    paragraphs = []
    i = 1
    styleNamePrefix = styleNamePrefix.strip() + " "
    
    while(True):
        styleId = getStyleIdByName(styleNamePrefix + str(i), styleXml)
        if styleId is None:
            break
        else:
            paragraphs += getParagraphElementsByStyleId(docXml, styleId)
            i += 1
    return paragraphs
    
def checkIndex(styles):
    '''Checks that the document has an automatically made index.'''

    indexParagraphs = getParagraphElementsBySequentialStyleName("index ", styles)
    if len(indexParagraphs) == 0: return False
    if len(indexParagraphs) != 0: return True
    #TODO: if index is automatically made -> check if index is correctly made (below)
    
    # The previous w:p element of the first index entry should be something like this:
    #<w:p w:rsidR="002F2A09" w:rsidRDefault="00CA51D5">
        #<w:pPr>
            #<w:sectPr w:rsidR="002F2A09" w:rsidSect="002F2A09"> --- </w:sectPr>
        #</w:pPr>
        #<w:r>
            #<w:fldChar w:fldCharType="begin"/>
        #</w:r>
        #<w:r>
            #<w:instrText xml:space="preserve"> INDEX \c "2" \z "1035" </w:instrText>
        #</w:r>
        #<w:r>
            #<w:fldChar w:fldCharType="separate"/>
        #</w:r>
    #</w:p>
    
    # Search the 'instrText' field element and be sure that it's an index field element.
    try:
        indexFieldCodeElement = indexParagraphs[0].previousSibling.getElementsByTagName('w:instrText')[0]
    except:
        indexFieldCodeElement = None
    # There can be a section brake between the first index entry and the field declaration?
    if indexFieldCodeElement is None:
        try:
            indexFieldCodeElement = indexParagraphs[0].previousSibling.previousSibling.getElementsByTagName('w:instrText')[0]
        except:
            indexFieldCodeElement = None        
    
    if indexFieldCodeElement is None:
        errors.append('Index is not a field - make the index automatically, not manually.')
    elif getTextContent(indexFieldCodeElement).find('INDEX') == -1:
        errors.append('INDEX-text not found in field declaration - make the index automatically, not manually.')
    
    #return True
        
def checkIndexContent(styles):
    ''' Checks that the document has a index that is not empty, and that the index entries are referenced somewhere in the document.
    First gets all the index styles' definitions from styles.xml and finds paragraphs using the styles in the document.xml.
    Checks that there is a field code element indicating that the index is generated automatically.
    Collects the content of the index and checks it isn't empty.
    Finds references to the index entries and matches them to the index content.
    
    Index example:
    <w:p w:rsidR="002F2A09" w:rsidRDefault="00CA51D5">
        <w:r>
            <w:fldChar w:fldCharType="begin"/>
        </w:r>
        <w:r>
            <w:instrText xml:space="preserve"> INDEX \c "2" \z "1035" </w:instrText>
        </w:r>
        <w:r>
            <w:fldChar w:fldCharType="separate"/>
        </w:r>
    </w:p>

    <w:p w:rsidR="002F2A09" w:rsidRDefault="002F2A09">
        −
        <w:pPr>
            <w:pStyle w:val="Index1"/>
            <w:tabs>
                <w:tab w:val="right" w:leader="dot" w:pos="3950"/>
            </w:tabs>
        </w:pPr>
        <w:r>
            <w:t>Index entry level 1</w:t>
        </w:r>
    </w:p>
    
    Reference example:
    <w:r w:rsidR="00B27B47">
        <w:instrText xml:space="preserve"> XE "</w:instrText>
    </w:r>
    <w:r w:rsidR="00B27B47" w:rsidRPr="00B27B47">
        <w:instrText>Level 1 entry</w:instrText>
    </w:r>
    <w:r w:rsidR="00B27B47" w:rsidRPr="00B27B47">
        <w:instrText>:</w:instrText>
    </w:r>
    <w:r w:rsidR="00B27B47" w:rsidRPr="0011587C">
        <w:instrText>Level 2 entry</w:instrText>
    </w:r>

    '''
    indexParagraphs  = getParagraphElementsBySequentialStyleName("index ", styles)
    indexTextContent = dict()
    for p in indexParagraphs:
        textContent = getTextContent(p)
        if textContent is not None and textContent != "":
            indexTextContent[textContent] = None
    if len(indexTextContent) == 0:
        errors.append('Index is empty.')
    
    documentFieldTexts = ""
    for pElement in doc.getElementsByTagName('w:instrText'):
        documentFieldTexts += getTextContent(pElement)
    
    # Index entry reference example: 'XE "MainEntry"', 'XE "MainEntry:SubEntry"' or even 'XE "MainEntry:Heading" "Subentry:Heading"'
    # Check that the entries are actually included in the index:
    # Parse the string containing all text content of the w:instrText-elements.
    # First split at XE_ (where _ is whitespace), next split at \" and finally split at \:.
    # Compare the final index entry candidate to the index entries and visa versa to see if they match.
    # If finally some index entry doesn't have any matches, the entry is probably made manually. It's referenced nowhere!
    # TODO: check that the reference makes actually sense? Page number to the index comes from the page where the reference is.

    indexReferenceFieldsContent = []
    for field in documentFieldTexts.split('XE '):
        for candidate in field.split('\"'):
            for finalCandidate in candidate.split(":"):
                if finalCandidate.strip() != "":
                    indexReferenceFieldsContent.append(finalCandidate)
    
                    
    for indexReferenceComponent in indexReferenceFieldsContent:
        #print indexReferenceComponent
        for key in indexTextContent.keys():
            if key.find(indexReferenceComponent) != -1:
                indexTextContent[key] = True
                break
            elif indexReferenceComponent.find(key) != -1:
                indexTextContent[key] = True
                break

    for key in indexTextContent.keys():
        #print key + " - " + str(indexTextContent[key])
        if indexTextContent[key] is None:
            errors.append('No references found for index entry ' + key + '.')
    
def checkStringFromDocument(string):
    '''Checks if a string is found in the text content of the document (in the w:t-elements).
    If string is found, gives an error about how many occurences were found in a paragraph.
    
    return -- True if string is found in the document, False otherwise. 
    '''
    found = False
    for p in doc.getElementsByTagName('w:p'):
        textContent = ""
        for textElement in p.getElementsByTagName('w:t'):
            textContent += getTextContent(textElement)
        occurrences = textContent.count(string)
        if occurrences > 0:
            errors.append("\"" + string + "\" occurs " + str(occurrences) + " time(s) in paragraph: " + textContent[:25])
            found = True
    return found
#    for p in doc.getElementsByTagName('w:p'):
#        if(checkStringFromContent(p, "  ")):
#            errors.append("Double whitespace in paragraph: " + getTextContent(p))

def checkTabs():
    ''' Checks that tabulator is not used in the document.
    Exceptions: automatically generated table of contents and index contain tabulators.
    '''
    #TODO: More exceptions?
    #TODO: If a tab is found get the contents of w:t -elements to prevent showing meta fields' information.
    
    tabParagraphs = doc.getElementsByTagName('w:tab')
    if len(tabParagraphs) == 0:
        return False
    
    #tabParagraphContent = []
    tabParagraphContent = dict()
    
    for tab in tabParagraphs:
        tabParent = getParentParagraph(tab)
    # FIXME: add try - except. 
        tabParentStyleId = tabParent.getElementsByTagName('w:pStyle')[0].getAttribute('w:val')
        if getStyleNameById(tabParentStyleId).startswith('toc') or \
           getStyleNameById(tabParentStyleId).startswith('index'):
            continue
        else:
            #tabParagraphContent.append(getTextContent(tabParent))
            try:
                tabParagraphContent[getTextContent(tabParent)] += 1
            except KeyError:
                tabParagraphContent[getTextContent(tabParent)] = 1
            
    if len(tabParagraphContent) == 0:
        return False
    
    return tabParagraphContent

def isParagraphEmpty(p):
    
    #FIXME: not working yet.
    pContent = getTextFromParagraph(p).strip()
    if len(pContent) == 0:
        if len(p.getElementsByTagName('pic:pic')) > 0:
            return False
        if len(p.getElementsByTagName('w:sectPr')) > 0:
            return False
        if len(p.getElementsByTagName('w:pict')) > 0:
            return False
        if len(p.getElementsByTagName('w:object')) > 0:
            return False
        if len(p.getElementsByTagName('a:graphic')) > 0:
            return False
        if getParentParagraph(p, 'w:tbl') is not None:
            return False
        if p.previousSibling is not None:
            if p.previousSibling.tagName == 'w:tbl':
                return False
            styleId = getParagraphStyleId(p.previousSibling)
            if styleId is not None:
                styleName = getStyleNameById(styleId)
                if styleName is not None:
                    if styleName.find('toc') != -1:
                        return False
        if p.previousSibling is not None:
            styleId = getParagraphStyleId(p.previousSibling)
            if styleId is not None:
                styleName = getStyleNameById(styleId)
                if styleName is not None:
                    if styleName.find('index') != -1:
                        return False
        if p.previousSibling.previousSibling is not None:
            styleId = getParagraphStyleId(p.previousSibling.previousSibling)
            if styleId is not None:
                styleName = getStyleNameById(styleId)
                if styleName is not None:
                    if styleName.find('index') != -1:
                        return False
                
        #print getTextContent(p)
        #print getTextContent(p.previousSibling)
        #print getTextContent(p.previousSibling.previousSibling)
        return True
    return False

def findEmptyParagraphs():
    ''' Finds all empty paragraphs in the document.
    EXCEPTIONS: Picture in the document produces an empty paragraph.
                Empty table cell produces an empty paragraph.
                A table produces an empty paragraph right after the table.
                ...?
                
    return -- a dict with previous or next paragraph text as a key and count as value.
    '''
    paragraphs = doc.getElementsByTagName('w:p')
    emptyParagraphs = dict()
    
    for p in paragraphs:
            result = isParagraphEmpty(p)
            
            if result is True:
                paragraphBefore = p.previousSibling
                paragraphAfter = p.nextSibling

                
                
                
                if paragraphBefore is not None and getTextFromParagraph(paragraphBefore).strip() != "":
                    emptyParagraphs[getTextFromParagraph(paragraphBefore)] = 1
                elif paragraphAfter is not None and getTextFromParagraph(paragraphAfter).strip() != "":
                    emptyParagraphs[getTextFromParagraph(paragraphAfter)] = 1

#                try:
#                    emptyParagraphs[getTextTagsFromParagraphs(paragraphAfter)] += 1
#                except KeyError:
#                    emptyParagraphs[getTextTagsFromParagraphs(paragraphAfter)] = 1
                else:
                    try:
                        emptyParagraphs[""] += 1
                    except KeyError:
                        emptyParagraphs[""] = 1
    return emptyParagraphs

def checkList(document, list='List'):
    ''' Goes through all paragraph-elements in the document looking for paragraphs that use a list style.
    
    return -- True, if a list style is used in the document, False otherwise.
    '''
    docXml = document['word/document.xml']
    
    #TODO: iffittele list-tyyppi, onko esim. list numbered vai numbered list
    
    for p in docXml.getElementsByTagName('w:p'):
        styleId = getParagraphStyleId(p)
        if styleId is not None:
            styleName = getStyleNameById(styleId)
            if styleName.find(list) != -1:
                return True
    
    return False

def checkSpreadsheetChart():
    '''Check that the document has a chart copied from a spreadsheet document.
    Chart must be pasted as a link.
    '''
    
    objectElements = doc.getElementsByTagName('w:object')
    if len(objectElements) == 0:
        return False
    
    for object in objectElements:
        if len(object.getElementsByTagName('v:formulas')) > 0:
            
            try:
                OLEObjectElement = object.getElementsByTagName('o:OLEObject')[0]
            except:
                continue

            if OLEObjectElement.getAttribute('ProgID').find('Excel') != -1:
                #print "Spreadsheet chart is not from Excel."
                #print OLEObjectElement.getAttribute('Type')
                if OLEObjectElement.getAttribute('Type') == 'Link':
                    
                    rid = OLEObjectElement.getAttribute('r:id')
                    target = getRelsTargetByRId(rid, docRelsXml)
                    targetChart = target.split("!")
                    targetChart.reverse()
                    #Example: targetChart[0] = %5bmalli.xlsx%5dmalli%20Chart%201
                    targetChartName = targetChart[0]
                    
                    #TODO: more effective examination required, just check that there is more than three %-characters.
                    if targetChartName.count('%') < 3:
                        return False
                    
                    #print targetChartName
                    
                    
                    return True
                else:
                    return "Spreadsheet object is not pasted as a link."
    return False

def checkSpreadsheetTable():
    '''Check that the document has a table copied from a spreadsheet document.
    For now checks that the table is pasted as a link.
    '''
    objectElements = doc.getElementsByTagName('w:object')
    if len(objectElements) == 0:
        return False
    
    for object in objectElements:
        if len(object.getElementsByTagName('v:formulas')) == 0:
            
            try:
                OLEObjectElement = object.getElementsByTagName('o:OLEObject')[0]
            except:
                continue

            if OLEObjectElement.getAttribute('ProgID').find('Excel') != -1:
                #print "Spreadsheet chart is not from Excel."
                #print OLEObjectElement.getAttribute('Type')
                if OLEObjectElement.getAttribute('Type') == 'Link':
                    rid = OLEObjectElement.getAttribute('r:id')
                    target = getRelsTargetByRId(rid, docRelsXml)
                    targetChart = target.split("!")
                    targetChart.reverse()
                    targetTableCells = targetChart[0]
                    
                    #Example: targetTableCells = R1C1:R7C4
                    #TODO: more effective examination might be required, just check that 2 R- and 2 C-characters are found.
                    if targetTableCells.count('R') != 2:
                        return False
                    if targetTableCells.count('C') != 2:
                        return False
                    #print targetTableCells
                    
                    return True
                else:
                    return "Spreadsheet object is not pasted as a link."
    return False

def checkPresentationGraphicsChart():
    '''Check that the document contains a chart pasted from PowerPoint as a vector graphics picture or as an object.
    Doesn't really know if the picture or object is actually from PowerPoint!!
    '''
    pictureTargets = getPicturePaths(docRelsXml)
    for target in pictureTargets:
        if target.endswith('.emf') is True:
            # .emf vector graphics picture was found.
            return True
        if target.endswith('wmf') is True:
            return True
    
    
    # TODO: check the object if no vector graphics picture is found.
    
    # Normal jpg or png pictures may be inside the w:drawing-element.
    #drawingElements = doc.getElementsByTagName('w:drawing')
    #if len(drawingElements) > 0:
    #    return True
    
    return False


if __name__ == "__main__":
    '''For testing.'''
    #filename = 'sampleFiles/docx/miksi_kannattaa_opiskella_matematiikkaa.docx'
    #filename = 'sampleFiles/docx/Wordteksti2007.docx' - testaa getSectionProperties, tässä muuttuu!
    #filename = 'sampleFiles/docx/Harjoitustyo_elina_asikainen.docx'
    #filename = 'sampleFiles/docx/Teija_Holtta.docx' # asetus19.xml, see below
    #filename = 'sampleFiles/docx/Harjoitustyo2.docx'
    #filename = 'sampleFiles/docx/HarjoitustyoD.docx'
    #filename = 'sampleFiles/docx/Harjoitustyo.docx'
    #filename = 'sampleFiles/docx/wordi.docx'
    #filename = 'sampleFiles/docx/sivunumerointi_oikein_hdr_ftr.docx'
    #filename = 'sampleFiles/docx/vilin.docx'
    #filename = 'sampleFiles/docx/sivunumerointi_kasin.docx'
    #filename = 'sampleFiles/docx/tekstinkasittely.docx'
    #filename = 'sampleFiles/docx/teemakokeilu.docx'
    #filename = 'sampleFiles/docx/docx_demo.docx'
    #filename = 'sampleFiles/docx/otsikko_numerointi_ei_otsikko1.docx'
    #filename = 'sampleFiles/docx/index_lorem1.docx'
    #filename = 'sampleFiles/docx/index_lorem2.docx'
    #filename = 'sampleFiles/docx/hakemisto_kasin.docx'
    #filename = 'sampleFiles/docx/index_lorem2_whitespace_asterix.docx'
    #filename = 'sampleFiles/docx/otsikko2ja3_ei_numerointia.docx'
    #filename = 'sampleFiles/docx/otsikko2ja3_numerointi.docx'
    #filename = 'sampleFiles/docx/otsikko2ja3_numerointi_poistettu.docx'
    #filename = 'sampleFiles/docx/otsikko2ja4_numerointi.docx'
    #filename = 'sampleFiles/docx/empty_paragraphs.docx'
    #filename = 'sampleFiles/docx/empty_paragraphs_breaks.docx'
    #filename = 'sampleFiles/docx/empty_paragraphs_breaks_1lesspara.docx'
    #filename = 'sampleFiles/docx/taulukkolaskenta_esitysgrafiikka_paste.docx'
    filename = 'sampleFiles/docx/taulukkolaskenta_esitysgrafiikka_paste2007.docx'
    zip = zipfile.ZipFile(filename)
    filelist = zip.infolist()
    docFile = zip.read('word/document.xml')
    doc = xml.dom.minidom.parseString(docFile)
    coreFile = zip.read('docProps/core.xml')
    coreXml = xml.dom.minidom.parseString(coreFile)
    styleFile = zip.read('word/styles.xml')
    styleXml = xml.dom.minidom.parseString(styleFile)
    docRelsFile = zip.read('word/_rels/document.xml.rels')
    docRelsXml = minidom.parseString(docRelsFile)
    themeFile = zip.read('word/theme/theme1.xml')
    themeXml = minidom.parseString(themeFile)
    #requirements = Requirements('sampleFiles/xml/asetus19.xml') # Teija Hölttä -testitapaus
    #requirements.printRequirements(requirements.normal)
    
#    style = getCompleteStyleDefinitions(styleXml, 'Body Text')
#    printDict(style)
#    print("\n")
    #style = getCompleteStyleDefinitions(styleXml, 'Normal')
    
    style = getStyle(docFile, "heading 1")
    printDict(style)
    #print("\n")
    #print getPageSize()
    #print getPageMarginals()
#    style = getCompleteStyleDefinitions(styleXml, "heading 2")
#    printDict(style)
#    print("\n")
    
#    print("Page properites:")
#    sectPrps = getSectionProperties()
#    printDict(sectPrps)
#    print '\n'
    
    #sectPrps = getSectionProperties()
    #printDict(sectPrps)
    #for item in style.keys():
    #    print item + ": " + str(style[item])
        
    #print "\n"
    
    #checkStyleUsage()
    
    #checkTocContent()
    #checkIfTocIsCorrect()
    
    #print getTextContent(doc)
    #print "Endnote: " + str(checkEndnote())
    #print "Footnote: " + str(checkFootnote())
    
    #print "Pictures: " + str(checkImages({"word/document.xml":doc}))
    #print "Caption: " + str(checkPictureCaptions())
    #print "Cross reference: " + str(checkCrossRefrenceToPictureCaption())
    
    checkHeadingNumbering(zip)
    #checkIndex(styleXml)
    #checkIndexContent(styleXml)
    #checkHeadingNumbering(zip)
    #checkIndex()
    #print "Kaksi välilyöntiä: " + str(checkStringFromDocument("  "))
    #print "Tähtimerkki : " + str(checkStringFromDocument("*"))
    #print "Tabulaattoreita: " + str(checkTabs())
    print "Empty paragraphs after these paragraphs: " + str(findEmptyParagraphs())
    #print "List-style used: " + str(checkList())
    print "Spreadsheet chart: " + str(checkSpreadsheetChart())
    print "Spreadsheet table: " + str(checkSpreadsheetTable())
    print "Vector graphics picture or drawing object in the document: " + str(checkPresentationGraphicsChart())
    
    print "\nERRORS: "
    #errors.sort(cmp=None, key=None, reverse=False)
    for item in errors:
        print item
