#!/usr/bin/python
# -*- coding: UTF-8 -*-
'''
Created on 7.3.2011

@author: olkauppi
'''
import xml.dom.minidom

class Requirements:
    
    def __init__(self, requirementsFile):
        
        self.requirements = self.getRequirements(requirementsFile)
        
        
    def getRequirements(self, requirementsFile):
        reqs = []
        for requirement in requirementsFile.getElementsByTagName('requirement'):
            requirementObject = Requirement(requirement)
            reqs.append(requirementObject)
            
        return reqs
        

class Requirement:
    '''
    '''

    def __init__(self, requirement):
        ''' Initialize a new Requirement-object.
        '''
        # Use the requirement name to guide it to the correct word_processing function.
        self.name = requirement.getAttribute('name')
        self.category = requirement.getAttribute('category')
        self.expectedValue = self.getExpectedValues(requirement)        
        self.feedback = self.getFeedbacks(requirement)

    def getErrorMessage(self, errorValue = "DEFAULT", propertyName = None ):
        #errorMessage = ""
        
        
        return self.feedback[errorValue]
    
    def getExpectedValues(self, requirement):
        expectedElements = requirement.getElementsByTagName('expectedvalue')
        if len(expectedElements) == 1:
            return self._bool(expectedElements[0].firstChild.nodeValue)
            #return True
        
        properties = {}
        for property in requirement.getElementsByTagName('expectedvalue'):
            properties[property.getAttribute('name')] = property.firstChild.nodeValue
        return properties
            
    def getFeedbacks(self, requirement):
        feedbacks = {}
        for feedback in requirement.getElementsByTagName('feedback'):
            errorvalue = feedback.getElementsByTagName('errorvalue')[0].firstChild.nodeValue
            text = feedback.getElementsByTagName('text')[0].firstChild.nodeValue
            feedbacks[errorvalue] = text
        return feedbacks

        
    def _bool(self, s):
        return s.upper() == 'TRUE'
#def main():    
#    printRequirements ('normal')
#    printRequirements ('h1')
#    printRequirements ('h2')
#    printRequirements ('powerpoint')
#    printRequirements ('listbullet')
#    printRequirements ('listbullet2')
#    printRequirements ('listnumber')
    
#    def getRequirements(self, styleName, styleAttribute):
#        styleNode = self.requirementsDom.getElementsByTagName (styleName)[0]
#        attributeNode = styleNode.getElementsByTagName (styleAttribute)
#        if attributeNode:
#            attributeValue = attributeNode[0].firstChild.nodeValue
#        else:
#            return ''
#        return attributeValue
#    
#    
#    def printRequirements (self, style):
#        print style, '-vaatimus'
#        print 'englanti:', self.getRequirements(style,'englanti')
#        print 'suomi:', self.getRequirements(style,'suomi')
#        print 'kirjasin:', self.getRequirements(style,'kirjasin')
#        print 'kirjasinkoko:', self.getRequirements(style,'kirjasinkoko')
#        print 'korostus:', self.getRequirements(style,'korostus')
#        print 'sisennys:', self.getRequirements(style,'sisennys')
#        print 'sarkain:', self.getRequirements(style,'sarkain')
#        print 'rivivali:', self.getRequirements(style,'rivivali')
#        print 'tasaus:', self.getRequirements(style,'tasaus')
#        print 'valiennen:', self.getRequirements(style,'valiennen')
#        print 'valijalkeen', self.getRequirements(style,'valijalkeen')
#        print 'listapohja:', self.getRequirements(style,'listapohja')
#        print 'symboli:', self.getRequirements(style,'symboli')
#        print ''


if __name__ == "__main__":
    filename = 'sampleFiles/xml/requirements_test1.xml'
    #file = open(filename)
    
    reqDom = xml.dom.minidom.parse(filename)
    reqs = Requirements(reqDom)
    #print reqs.requirements
    for req in reqs.requirements:
        print req.name
        #print req.expectedValue
        print req.expectedValue
            
        print req.getErrorMessage()


